@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: classdef.h,v 1.39 2001/03/19 19:27:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,28 +18,28 @@
 #ifndef CLASSDEF_H
 #define CLASSDEF_H
 
-#include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
 #include <qptrdict.h>
-#include <qstrlist.h>
 
-#include "util.h"
-#include "memberlist.h"
 #include "definition.h"
-#include "sortdict.h"
 
+struct Argument;
+class MemberDef;
+class MemberList;
 class MemberDict;
 class ClassList;
 class ClassSDict;
 class OutputList;
 class FileDef;
+class FileList;
 class BaseClassList;
 class NamespaceDef;
 class MemberDef;
 class ExampleSDict;
 class MemberNameInfoSDict;
 class UsesClassDict;
+class ConstraintClassDict;
 class MemberGroupSDict;
 class QTextStream;
 class PackageDef;
@@ -47,23 +47,28 @@ class GroupDef;
 class StringDict;
 struct IncludeInfo;
 class ClassDefImpl;
+class ArgumentList;
+class FTextStream;
 
-/** A model of a compound symbol.
+/** A class representing of a compound symbol.
  *
- *  A compound can be a class, struct, union, interface, or exception.
+ *  A compound can be a class, struct, union, interface, service, singleton,
+ *  or exception.
  *  \note This class should be renamed to CompoundDef
  */
 class ClassDef : public Definition
 {
   public:
     /** The various compound types */
-    enum CompoundType { Class, //=Entry::CLASS_SEC, 
-                        Struct, //=Entry::STRUCT_SEC, 
-                        Union, //=Entry::UNION_SEC,
+    enum CompoundType { Class,     //=Entry::CLASS_SEC, 
+                        Struct,    //=Entry::STRUCT_SEC, 
+                        Union,     //=Entry::UNION_SEC,
                         Interface, //=Entry::INTERFACE_SEC,
-                        Protocol, //=Entry::PROTOCOL_SEC,
-                        Category, //=Entry::CATEGORY_SEC,
-                        Exception //=Entry::EXCEPTION_SEC
+                        Protocol,  //=Entry::PROTOCOL_SEC,
+                        Category,  //=Entry::CATEGORY_SEC,
+                        Exception, //=Entry::EXCEPTION_SEC
+                        Service,   //=Entry::CLASS_SEC
+                        Singleton, //=Entry::CLASS_SEC
                       };
 
     /** Creates a new compound definition.
@@ -71,6 +76,8 @@ class ClassDef : public Definition
      *                   found.
      *  \param startLine line number where the definition of this compound
      *                   starts.
+     *  \param startColumn column number where the definition of this compound
+     *                   starts.
      *  \param name      the name of this compound (including scope)
      *  \param ct        the kind of Compound
      *  \param ref       the tag file from which this compound is extracted
@@ -84,7 +91,7 @@ class ClassDef : public Definition
      *                    I didn't add this to CompoundType to avoid having
      *                    to adapt all translators.
      */
-    ClassDef(const char *fileName,int startLine,
+    ClassDef(const char *fileName,int startLine,int startColumn,
              const char *name,CompoundType ct,
              const char *ref=0,const char *fName=0,
              bool isSymbol=TRUE,bool isJavaEnum=FALSE);
@@ -101,7 +108,6 @@ class ClassDef : public Definition
     /** Returns the unique base name (without extension) of the class's file on disk */
     QCString getOutputFileBase() const;
     QCString getInstanceOutputFileBase() const; 
-    QCString getFileBase() const;
 
     /** Returns the base name for the source code file */
     QCString getSourceFileBase() const; 
@@ -116,11 +122,20 @@ class ClassDef : public Definition
     bool isLocal() const;
 
     /** returns the classes nested into this class */
-    ClassSDict *getClassSDict();
+    ClassSDict *getClassSDict() const;
 
     /** returns TRUE if this class has documentation */
     bool hasDocumentation() const;
 
+    /** returns TRUE if this class has a non-empty detailed description */
+    bool hasDetailedDescription() const;
+    
+    /** returns the file name to use for the collaboration graph */
+    QCString collaborationGraphFileName() const;
+
+    /** returns the file name to use for the inheritance graph */
+    QCString inheritanceGraphFileName() const;
+
     /** Returns the name as it is appears in the documentation */
     QCString displayName(bool includeScope=TRUE) const;
 
@@ -161,6 +176,9 @@ class ClassDef : public Definition
     /** the class is visible in a class diagram, or class hierarchy */
     bool isVisibleInHierarchy();
     
+    /** show this class in the declaration section of its parent? */
+    bool visibleInParentsDeclList() const;
+
     /** Returns the template arguments of this class 
      *  Will return 0 if not applicable.
      */
@@ -182,11 +200,16 @@ class ClassDef : public Definition
     MemberDef    *getMemberByName(const QCString &) const;
     
     /** Returns TRUE iff \a bcd is a direct or indirect base class of this
-     *  class. This function will recusively traverse all branches of the
+     *  class. This function will recursively traverse all branches of the
      *  inheritance tree.
      */
     bool isBaseClass(ClassDef *bcd,bool followInstances,int level=0);
 
+    /** Returns TRUE iff \a bcd is a direct or indirect sub class of this
+     *  class.
+     */
+    bool isSubClass(ClassDef *bcd,int level=0);
+
     /** returns TRUE iff \a md is a member of this class or of the
      *  the public/protected members of a base class 
      */
@@ -213,13 +236,15 @@ class ClassDef : public Definition
 
     UsesClassDict *usedInterfaceClasses() const;
 
+    ConstraintClassDict *templateTypeConstraints() const;
+
     bool isTemplateArgument() const;
 
     /** Returns the definition of a nested compound if
      *  available, or 0 otherwise.
      *  @param name The name of the nested compound
      */
-    virtual Definition *findInnerCompound(const char *name);
+    virtual Definition *findInnerCompound(const char *name) const;
 
     /** Returns the template parameter lists that form the template
      *  declaration of this class.
@@ -231,7 +256,7 @@ class ClassDef : public Definition
     void getTemplateParameterLists(QList<ArgumentList> &lists) const;
 
     QCString qualifiedNameWithTemplateParameters(
-        QList<ArgumentList> *actualParams=0) const;
+        QList<ArgumentList> *actualParams=0,int *actualParamIndex=0) const;
 
     /** Returns TRUE if there is at least one pure virtual member in this
      *  class.
@@ -241,6 +266,9 @@ class ClassDef : public Definition
     /** Returns TRUE if this class is implemented in Objective-C */
     bool isObjectiveC() const;
 
+    /** Returns TRUE if this class is implemented in Fortran */
+    bool isFortran() const;
+
     /** Returns TRUE if this class is implemented in C# */
     bool isCSharp() const;
 
@@ -250,9 +278,15 @@ class ClassDef : public Definition
     /** Returns TRUE if this class is marked as sealed */
     bool isSealed() const;
 
+    /** Returns TRUE if this class is marked as published */
+    bool isPublished() const;
+
     /** Returns TRUE if this class represents an Objective-C 2.0 extension (nameless category) */
     bool isExtension() const;
 
+    /** Returns TRUE if this class represents a forward declaration of a template class */
+    bool isForwardDeclared() const;
+
     /** Returns the class of which this is a category (Objective-C only) */
     ClassDef *categoryOf() const;
 
@@ -262,7 +296,7 @@ class ClassDef : public Definition
     QCString className() const;
 
     /** Returns the members in the list identified by \a lt */
-    MemberList *getMemberList(MemberList::ListType lt);
+    MemberList *getMemberList(MemberListType lt);
 
     /** Returns the list containing the list of members sorted per type */
     const QList<MemberList> &getMemberLists() const;
@@ -289,6 +323,20 @@ class ClassDef : public Definition
     bool isJavaEnum() const;
 
     bool isGeneric() const;
+    bool isAnonymous() const;
+    const ClassSDict *innerClasses() const;
+    QCString title() const;
+
+    QCString generatedFromFiles() const;
+    const FileList &usedFiles() const;
+
+    const ArgumentList *typeConstraints() const;
+    const ExampleSDict *exampleList() const;
+    bool hasExamples() const;
+    QCString getMemberListFileName() const;
+    bool subGrouping() const;
+
+    bool isSliceLocal() const;
 
     //-----------------------------------------------------------------------------------
     // --- setters ----
@@ -298,7 +346,7 @@ class ClassDef : public Definition
     void insertSubClass(ClassDef *,Protection p,Specifier s,const char *t=0);
     void setIncludeFile(FileDef *fd,const char *incName,bool local,bool force); 
     void insertMember(MemberDef *);
-    void insertUsedFile(const char *);
+    void insertUsedFile(FileDef *);
     bool addExample(const char *anchor,const char *name, const char *file);
     void mergeCategory(ClassDef *category);
     void setNamespace(NamespaceDef *nd);
@@ -307,14 +355,14 @@ class ClassDef : public Definition
     void setProtection(Protection p);
     void setGroupDefForAllMembers(GroupDef *g,Grouping::GroupPri_t pri,const QCString &fileName,int startLine,bool hasDocs);
     void addInnerCompound(Definition *d);
-    ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,
+    ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
                                 const QCString &templSpec,bool &freshInstance);
     void addUsedClass(ClassDef *cd,const char *accessName,Protection prot);
     void addUsedByClass(ClassDef *cd,const char *accessName,Protection prot);
     void setIsStatic(bool b);
     void setCompoundType(CompoundType t);
     void setClassName(const char *name);
-    void setClassSpecifier(int spec);
+    void setClassSpecifier(uint64 spec);
 
     void setTemplateArguments(ArgumentList *al);
     void setTemplateBaseClassNames(QDict<int> *templateNames);
@@ -327,6 +375,9 @@ class ClassDef : public Definition
 
     void addTaggedInnerClass(ClassDef *cd);
     void setTagLessReference(ClassDef *cd);
+    void setName(const char *name);
+
+    void setMetaData(const char *md);
 
     //-----------------------------------------------------------------------------------
     // --- actions ----
@@ -335,6 +386,7 @@ class ClassDef : public Definition
     void findSectionsInDocumentation();
     void addMembersToMemberGroup();
     void addListReferences();
+    void addTypeConstraints();
     void computeAnchors();
     void mergeMembers();
     void sortMemberLists();
@@ -347,40 +399,40 @@ class ClassDef : public Definition
                           ClassDef *inheritedFrom,const char *inheritId);
     void writeQuickMemberLinks(OutputList &ol,MemberDef *md) const;
     void writeSummaryLinks(OutputList &ol);
-    void reclassifyMember(MemberDef *md,MemberDef::MemberType t);
+    void reclassifyMember(MemberDef *md,MemberType t);
     void writeInlineDocumentation(OutputList &ol);
     void writeDeclarationLink(OutputList &ol,bool &found,
                               const char *header,bool localNames);
     void removeMemberFromLists(MemberDef *md);
-    void addGroupedInheritedMembers(OutputList &ol,MemberList::ListType lt,
+    void addGroupedInheritedMembers(OutputList &ol,MemberListType lt,
                               ClassDef *inheritedFrom,const QCString &inheritId);
-    int countMembersIncludingGrouped(MemberList::ListType lt,ClassDef *inheritedFrom,bool additional);
+    int countMembersIncludingGrouped(MemberListType lt,ClassDef *inheritedFrom,bool additional);
+    int countInheritanceNodes();
+    void writeTagFile(FTextStream &);
     
     bool visited;
 
   protected:
     void addUsedInterfaceClasses(MemberDef *md,const char *typeStr);
-    bool hasExamples();
     bool hasNonReferenceSuperClass();
     void showUsedFiles(OutputList &ol);
 
   private: 
-    void writeTagFileMarker();
     void writeDocumentationContents(OutputList &ol,const QCString &pageTitle);
     void internalInsertMember(MemberDef *md,Protection prot,bool addToAllList);
-    QCString getMemberListFileName() const;
-    void addMemberToList(MemberList::ListType lt,MemberDef *md,bool isBrief);
-    MemberList *createMemberList(MemberList::ListType lt);
-    void writeInheritedMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,ClassDef *inheritedFrom,bool invert,QPtrDict<void> *visitedClasses);
-    void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,
-                                 const char *subTitle=0,bool showInline=FALSE,ClassDef *inheritedFrom=0,int lt2=-1,bool invert=FALSE,QPtrDict<void> *visitedClasses=0);
-    void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title,bool showInline=FALSE);
-    void writeSimpleMemberDocumentation(OutputList &ol,MemberList::ListType lt);
-    void writePlainMemberDeclaration(OutputList &ol,MemberList::ListType lt,bool inGroup,ClassDef *inheritedFrom,const char *inheritId);
+    void addMemberToList(MemberListType lt,MemberDef *md,bool isBrief);
+    MemberList *createMemberList(MemberListType lt);
+    void writeInheritedMemberDeclarations(OutputList &ol,MemberListType lt,int lt2,const QCString &title,ClassDef *inheritedFrom,bool invert,bool showAlways,QPtrDict<void> *visitedClasses);
+    void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title,
+                                 const char *subTitle=0,bool showInline=FALSE,ClassDef *inheritedFrom=0,int lt2=-1,bool invert=FALSE,bool showAlways=FALSE,QPtrDict<void> *visitedClasses=0);
+    void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title,bool showInline=FALSE);
+    void writeSimpleMemberDocumentation(OutputList &ol,MemberListType lt);
+    void writePlainMemberDeclaration(OutputList &ol,MemberListType lt,bool inGroup,ClassDef *inheritedFrom,const char *inheritId);
     void writeBriefDescription(OutputList &ol,bool exampleFlag);
     void writeDetailedDescription(OutputList &ol,const QCString &pageType,bool exampleFlag,
                                   const QCString &title,const QCString &anchor=QCString());
     void writeIncludeFiles(OutputList &ol);
+    void writeIncludeFilesForSlice(OutputList &ol);
     //void writeAllMembersLink(OutputList &ol);
     void writeInheritanceGraph(OutputList &ol);
     void writeCollaborationGraph(OutputList &ol);
@@ -395,16 +447,24 @@ class ClassDef : public Definition
     void writeMoreLink(OutputList &ol,const QCString &anchor);
     void writeDetailedDocumentationBody(OutputList &ol);
     
-    int countInheritedDecMembersRec(MemberList::ListType lt,ClassDef *inheritedFrom);
-    int countInheritedDecMembers(MemberList::ListType lt);
     int countAdditionalInheritedMembers();
     void writeAdditionalInheritedMembers(OutputList &ol);
     void addClassAttributes(OutputList &ol);
-    
-    ClassDefImpl *m_impl;
+    int countMemberDeclarations(MemberListType lt,ClassDef *inheritedFrom,
+                                int lt2,bool invert,bool showAlways,QPtrDict<void> *visitedClasses);
+    int countInheritedDecMembers(MemberListType lt,
+                                 ClassDef *inheritedFrom,bool invert,bool showAlways,
+                                 QPtrDict<void> *visitedClasses);
+    void getTitleForMemberListType(MemberListType type,
+               QCString &title,QCString &subtitle);
+    QCString includeStatement() const;
+    void addTypeConstraint(const QCString &typeConstraint,const QCString &type);
 
+    ClassDefImpl *m_impl;
 };
 
+//------------------------------------------------------------------------
+
 /** Class that contains information about a usage relation. 
  */
 struct UsesClassDef
@@ -458,6 +518,8 @@ class UsesClassDictIterator : public QDictIterator<UsesClassDef>
    ~UsesClassDictIterator() {}
 };
 
+//------------------------------------------------------------------------
+
 /** Class that contains information about an inheritance relation. 
  */
 struct BaseClassDef
@@ -496,14 +558,14 @@ class BaseClassList : public QList<BaseClassDef>
 {
   public:
    ~BaseClassList() {}
-    int compareItems(GCI item1,GCI item2)
+    int compareValues(const BaseClassDef *item1,const BaseClassDef *item2) const
     {
-      ClassDef *c1=((BaseClassDef *)item1)->classDef;
-      ClassDef *c2=((BaseClassDef *)item2)->classDef;
-      if (c1==0 || c2==0) 
+      const ClassDef *c1=item1->classDef;
+      const ClassDef *c2=item2->classDef;
+      if (c1==0 || c2==0)
         return FALSE;
       else
-        return stricmp(c1->name(),c2->name());
+        return qstricmp(c1->name(),c2->name());
     }
 };
 
@@ -516,4 +578,56 @@ class BaseClassListIterator : public QListIterator<BaseClassDef>
       QListIterator<BaseClassDef>(bcl) {}
 };
 
+//------------------------------------------------------------------------
+
+
+/** Class that contains information about a type constraint relations.
+ */
+struct ConstraintClassDef
+{
+  ConstraintClassDef(ClassDef *cd) : classDef(cd)
+  {
+    accessors = new QDict<void>(17);
+  }
+ ~ConstraintClassDef()
+  {
+    delete accessors;
+  }
+  void addAccessor(const char *s)
+  {
+    if (accessors->find(s)==0)
+    {
+      accessors->insert(s,(void *)666);
+    }
+  }
+  /** Class definition that this relation uses. */
+  ClassDef *classDef;
+
+  /** Dictionary of member types names that form the edge labels of the
+   *  constraint relation.
+   */
+  QDict<void> *accessors;
+};
+
+/** Dictionary of constraint relations.
+ */
+class ConstraintClassDict : public QDict<ConstraintClassDef>
+{
+  public:
+    ConstraintClassDict(int size) : QDict<ConstraintClassDef>(size) {}
+   ~ConstraintClassDict() {}
+};
+
+/** Iterator class to iterate over a dictionary of constraint relations.
+ */
+class ConstraintClassDictIterator : public QDictIterator<ConstraintClassDef>
+{
+  public:
+    ConstraintClassDictIterator(const QDict<ConstraintClassDef> &d)
+      : QDictIterator<ConstraintClassDef>(d) {}
+   ~ConstraintClassDictIterator() {}
+};
+
+//------------------------------------------------------------------------
+
 #endif
