@@ -3,7 +3,7 @@
  * 
  *
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,6 +18,13 @@
 
 #include "cmdmapper.h"
 
+/** Call representing a mapping from a command name to a command ID. */
+struct CommandMap
+{
+  const char *cmdName;
+  int cmdId;
+};
+
 CommandMap cmdMap[] =
 {
   { "a",             CMD_EMPHASIS },
@@ -29,10 +36,12 @@ CommandMap cmdMap[] =
   { "authors",       CMD_AUTHORS },
   { "b",             CMD_BOLD },
   { "c",             CMD_CODE },
+  { "cite",          CMD_CITE },
   { "code",          CMD_STARTCODE },
   { "copydoc",       CMD_COPYDOC },
   { "copybrief",     CMD_COPYBRIEF },
   { "copydetails",   CMD_COPYDETAILS },
+  { "copyright",     CMD_COPYRIGHT },
   { "date",          CMD_DATE },
   { "dontinclude",   CMD_DONTINCLUDE },
   { "dotfile",       CMD_DOTFILE },
@@ -54,6 +63,7 @@ CommandMap cmdMap[] =
   { "internal",      CMD_INTERNAL },
   { "invariant",     CMD_INVARIANT },
   { "javalink",      CMD_JAVALINK },
+  { "latexinclude",  CMD_LATEXINCLUDE },
   { "latexonly",     CMD_LATEXONLY },
   { "li",            CMD_LI },
   { "line",          CMD_LINE },
@@ -73,9 +83,13 @@ CommandMap cmdMap[] =
   { "return",        CMD_RETURN },
   { "returns",       CMD_RETURN },
   { "retval",        CMD_RETVAL },
+  { "rtfonly",       CMD_RTFONLY },
   { "sa",            CMD_SA },
   { "secreflist",    CMD_SECREFLIST },
   { "section",       CMD_SECTION },
+  { "snippet",       CMD_SNIPPET },
+  { "snippetdoc",    CMD_SNIPPETDOC },
+  { "snippetlineno", CMD_SNIPWITHLINES },
   { "subpage",       CMD_SUBPAGE },
   { "subsection",    CMD_SUBSECTION },
   { "subsubsection", CMD_SUBSUBSECTION },
@@ -97,22 +111,44 @@ CommandMap cmdMap[] =
   { "\\",            CMD_BSLASH },
   { "@",             CMD_AT },
   { "<",             CMD_LESS },
+  { "=",             CMD_EQUAL },
   { ">",             CMD_GREATER },
   { "&",             CMD_AMP },
   { "$",             CMD_DOLLAR },
   { "#",             CMD_HASH },
   { "%",             CMD_PERCENT },
+  { "|",             CMD_PIPE },
+  { ".",             CMD_PUNT },
+  { "+",             CMD_PLUS },
+  { "-",             CMD_MINUS },
+  { "::",            CMD_DCOLON },
   { "\"",            CMD_QUOTE },
   { "_internalref",  CMD_INTERNALREF },
   { "dot",           CMD_DOT },
   { "msc",           CMD_MSC },
+  { "startuml",      CMD_STARTUML },
   { "enddot",        CMD_ENDDOT },
   { "endmsc",        CMD_ENDMSC },
+  { "enduml",        CMD_ENDUML },
   { "manonly",       CMD_MANONLY },
   { "endmanonly",    CMD_ENDMANONLY },
   { "includelineno", CMD_INCWITHLINES },
+  { "includedoc",    CMD_INCLUDEDOC },
   { "inheritdoc",    CMD_INHERITDOC },
   { "mscfile",       CMD_MSCFILE },
+  { "rtfonly",       CMD_RTFONLY },
+  { "endrtfonly",    CMD_ENDRTFONLY },
+  { "vhdlflow",      CMD_VHDLFLOW },
+  { "docbookonly",   CMD_DBONLY },
+  { "enddocbookonly",CMD_ENDDBONLY },
+  { "endinternal",   CMD_ENDINTERNAL },
+  { "parblock",      CMD_PARBLOCK },
+  { "endparblock",   CMD_ENDPARBLOCK },
+  { "diafile",       CMD_DIAFILE },
+  { "--",            CMD_NDASH },
+  { "---",           CMD_MDASH },
+  { "_setscope",     CMD_SETSCOPE },
+  { "emoji",         CMD_EMOJI },
   { 0,               0 },
 };
 
@@ -158,6 +194,9 @@ CommandMap htmlTagMap[] =
   { "h6",         HTML_H6 },
   { "span",       HTML_SPAN },
   { "div",        HTML_DIV },
+  { "blockquote", HTML_BLOCKQUOTE },
+  { "strike",     HTML_STRIKE },
+  { "u",          HTML_UNDERLINE },
 
   { "c",            XML_C },
   // { "code",       XML_CODE },  <= ambiguous <code> is also a HTML tag
@@ -181,6 +220,7 @@ CommandMap htmlTagMap[] =
   { "summary",      XML_SUMMARY },
   { "term",         XML_TERM },
   { "value",        XML_VALUE },
+  { "inheritdoc",   XML_INHERITDOC },
   { 0,              0 }
 };
 
@@ -189,6 +229,35 @@ CommandMap htmlTagMap[] =
 Mapper *Mappers::cmdMapper     = new Mapper(cmdMap,TRUE);
 Mapper *Mappers::htmlTagMapper = new Mapper(htmlTagMap,FALSE);
 
+int Mapper::map(const char *n)
+{
+  QCString name=n;
+  if (!m_cs) name=name.lower();
+  int *result;
+  return !name.isEmpty() && (result=m_map.find(name)) ? *result: 0;
+}
+
+QString Mapper::find(const int n)
+{
+  QDictIterator<int> mapIterator(m_map);
+  for (int *curVal = mapIterator.toFirst();(curVal = mapIterator.current());++mapIterator)
+  {
+    if (*curVal == n || (*curVal == (n | SIMPLESECT_BIT))) return mapIterator.currentKey();
+  }
+  return NULL;
+}
+
+Mapper::Mapper(const CommandMap *cm,bool caseSensitive) : m_map(89), m_cs(caseSensitive)
+{
+  m_map.setAutoDelete(TRUE);
+  const CommandMap *p = cm;
+  while (p->cmdName)
+  {
+    m_map.insert(p->cmdName,new int(p->cmdId));
+    p++;
+  }
+}
+
 void Mappers::freeMappers()
 {
   delete cmdMapper;     cmdMapper     = 0;
