@@ -3,7 +3,7 @@
  * 
  *
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -274,7 +274,7 @@ static void writeMapArea(FTextStream &t,ClassDef *cd,QCString relPath,
     QCString tooltip = cd->briefDescriptionAsTooltip();
     if (!tooltip.isEmpty())
     {
-      t << "title=\"" << tooltip << "\" ";
+      t << "title=\"" << convertToHtml(tooltip) << "\" ";
     }
     t << "alt=\"" << convertToXML(cd->displayName()); 
     t << "\" shape=\"rect\" coords=\"" << x << "," << y << ",";
@@ -321,7 +321,7 @@ QCString DiagramItem::label() const
   {
     result=classDef->displayName();
   }
-  if (Config_getBool("HIDE_SCOPE_NAMES")) result=stripScope(result);
+  if (Config_getBool(HIDE_SCOPE_NAMES)) result=stripScope(result);
   return result;
 }
 
@@ -833,7 +833,7 @@ void TreeDiagram::drawConnectors(FTextStream &t,Image *image,
             }
             ++rit; di=rit.current();
           }
-          // add last horizonal line and a vertical connection line
+          // add last horizontal line and a vertical connection line
           if (bitmap)
           {
             if (doBase) // base classes
@@ -1332,7 +1332,7 @@ void ClassDiagram::writeFigure(FTextStream &output,const char *path,
   super->drawConnectors(t,0,FALSE,FALSE,baseRows,superRows,0,0);
 
   f1.close();
-  if (Config_getBool("USE_PDFLATEX"))
+  if (Config_getBool(USE_PDFLATEX))
   {
     QCString epstopdfArgs(4096);
     epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
@@ -1380,7 +1380,5 @@ void ClassDiagram::writeImage(FTextStream &t,const char *path,
 #define IMAGE_EXT ".png"
   image.save((QCString)path+"/"+fileName+IMAGE_EXT);
   Doxygen::indexList->addImageFile(QCString(fileName)+IMAGE_EXT);
-  
-  if (generateMap) t << "</map>" << endl;
 }
 
