@@ -20,7 +20,7 @@ static int g_dirCount=0;
 
 DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
 {
-  bool fullPathNames = Config_getBool("FULL_PATH_NAMES");
+  bool fullPathNames = Config_getBool(FULL_PATH_NAMES);
   // get display name (stipping the paths mentioned in STRIP_FROM_PATH)
   // get short name (last part of path)
   m_shortName = path;
@@ -67,29 +67,34 @@ bool DirDef::isLinkable() const
 
 void DirDef::addSubDir(DirDef *subdir)
 {
-  m_subdirs.inSort(subdir);
+  m_subdirs.append(subdir);
   subdir->setOuterScope(this);
   subdir->m_parent=this;
 }
 
 void DirDef::addFile(FileDef *fd)
 {
-  m_fileList->inSort(fd);
+  m_fileList->append(fd);
   fd->setDirDef(this);
 }
 
+void DirDef::sort()
+{
+  m_subdirs.sort();
+  m_fileList->sort();
+}
+
 static QCString encodeDirName(const QCString &anchor)
 {
-  QCString result;
-
   // convert to md5 hash
   uchar md5_sig[16];
   QCString sigStr(33);
   MD5Buffer((const unsigned char *)anchor.data(),anchor.length(),md5_sig);
-  MD5SigToString(md5_sig,sigStr.data(),33);
+  MD5SigToString(md5_sig,sigStr.rawData(),33);
   return sigStr;
 
   // old algorithm
+//  QCString result;
 
 //  int l = anchor.length(),i;
 //  for (i=0;i<l;i++)
@@ -120,7 +125,7 @@ QCString DirDef::getOutputFileBase() const
 
 void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) || 
+  if ((!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF)) || 
       !documentation().isEmpty())
   {
     ol.pushGeneratorState();
@@ -136,12 +141,12 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
     ol.endGroupHeader();
 
     // repeat brief description
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
+    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
       ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
     }
     // separator between brief and details
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
+    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) && 
         !documentation().isEmpty())
     {
       ol.pushGeneratorState();
@@ -165,20 +170,24 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 
 void DirDef::writeBriefDescription(OutputList &ol)
 {
-  if (!briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
+  if (hasBriefDescription())
   {
     DocRoot *rootNode = validatingParseDoc(
          briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
     if (rootNode && !rootNode->isEmpty())
     {
       ol.startParagraph();
+      ol.pushGeneratorState();
+      ol.disableAllBut(OutputGenerator::Man);
+      ol.writeString(" - ");
+      ol.popGeneratorState();
       ol.writeDoc(rootNode,this,0);
       ol.pushGeneratorState();
       ol.disable(OutputGenerator::RTF);
       ol.writeString(" \n");
       ol.enable(OutputGenerator::RTF);
 
-      if (Config_getBool("REPEAT_BRIEF") ||
+      if (Config_getBool(REPEAT_BRIEF) ||
           !documentation().isEmpty()
          )
       {
@@ -199,7 +208,7 @@ void DirDef::writeBriefDescription(OutputList &ol)
 void DirDef::writeDirectoryGraph(OutputList &ol)
 {
   // write graph dependency graph
-  if (Config_getBool("DIRECTORY_GRAPH") && Config_getBool("HAVE_DOT"))
+  if (Config_getBool(DIRECTORY_GRAPH) && Config_getBool(HAVE_DOT))
   {
     DotDirDeps dirDep(this);
     if (!dirDep.isTrivial())
@@ -218,36 +227,48 @@ void DirDef::writeDirectoryGraph(OutputList &ol)
 
 void DirDef::writeSubDirList(OutputList &ol)
 {
+  int numSubdirs = 0;
+  QListIterator<DirDef> it(m_subdirs);
+  DirDef *dd;
+  for (it.toFirst();(dd=it.current());++it)
+  {
+    if (dd->hasDocumentation() || dd->getFiles()->count()>0)
+    {
+      numSubdirs++;
+    }
+  }
+
   // write subdir list
-  if (m_subdirs.count()>0)
+  if (numSubdirs>0)
   {
     ol.startMemberHeader("subdirs");
     ol.parseText(theTranslator->trDir(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    QListIterator<DirDef> it(m_subdirs);
-    DirDef *dd;
-    for (;(dd=it.current());++it)
+    for (it.toFirst();(dd=it.current());++it)
     {
-      ol.startMemberDeclaration();
-      ol.startMemberItem(dd->getOutputFileBase(),0);
-      ol.parseText(theTranslator->trDir(FALSE,TRUE)+" ");
-      ol.insertMemberAlign();
-      ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),0,dd->shortName());
-      ol.endMemberItem();
-      if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
+      if (dd->hasDocumentation() || dd->getFiles()->count()==0)
       {
-        ol.startMemberDescription(dd->getOutputFileBase());
-        ol.generateDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
-            FALSE, // indexWords
-            FALSE, // isExample
-            0,     // exampleName
-            TRUE,  // single line
-            TRUE   // link from index
-           );
-        ol.endMemberDescription();
+        ol.startMemberDeclaration();
+        ol.startMemberItem(dd->getOutputFileBase(),0);
+        ol.parseText(theTranslator->trDir(FALSE,TRUE)+" ");
+        ol.insertMemberAlign();
+        ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),0,dd->shortName());
+        ol.endMemberItem();
+        if (!dd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
+        {
+          ol.startMemberDescription(dd->getOutputFileBase());
+          ol.generateDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
+              FALSE, // indexWords
+              FALSE, // isExample
+              0,     // exampleName
+              TRUE,  // single line
+              TRUE   // link from index
+              );
+          ol.endMemberDescription();
+        }
+        ol.endMemberDeclaration(0,0);
       }
-      ol.endMemberDeclaration(0,0);
     }
 
     ol.endMemberList();
@@ -256,8 +277,19 @@ void DirDef::writeSubDirList(OutputList &ol)
 
 void DirDef::writeFileList(OutputList &ol)
 {
+  int numFiles = 0;
+  QListIterator<FileDef> it(*m_fileList);
+  FileDef *fd;
+  for (it.toFirst();(fd=it.current());++it)
+  {
+    if (fd->hasDocumentation())
+    {
+      numFiles++;
+    }
+  }
+
   // write file list
-  if (m_fileList->count()>0)
+  if (numFiles>0)
   {
     ol.startMemberHeader("files");
     ol.parseText(theTranslator->trFile(TRUE,FALSE));
@@ -267,46 +299,49 @@ void DirDef::writeFileList(OutputList &ol)
     FileDef *fd;
     for (;(fd=it.current());++it)
     {
-      ol.startMemberDeclaration();
-      ol.startMemberItem(fd->getOutputFileBase(),0);
-      ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
-      ol.insertMemberAlign();
-      if (fd->isLinkable())
+      if (fd->hasDocumentation())
       {
-        ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),0,fd->name());
+        ol.startMemberDeclaration();
+        ol.startMemberItem(fd->getOutputFileBase(),0);
+        ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
+        ol.insertMemberAlign();
+        if (fd->isLinkable())
+        {
+          ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),0,fd->name());
+        }
+        else
+        {
+          ol.startBold();
+          ol.docify(fd->name());
+          ol.endBold();
+        }
+        if (fd->generateSourceFile())
+        {
+          ol.pushGeneratorState();
+          ol.disableAllBut(OutputGenerator::Html);
+          ol.docify(" ");
+          ol.startTextLink(fd->includeName(),0);
+          ol.docify("[");
+          ol.parseText(theTranslator->trCode());
+          ol.docify("]");
+          ol.endTextLink();
+          ol.popGeneratorState();
+        }
+        ol.endMemberItem();
+        if (!fd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
+        {
+          ol.startMemberDescription(fd->getOutputFileBase());
+          ol.generateDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
+              FALSE, // indexWords
+              FALSE, // isExample
+              0,     // exampleName
+              TRUE,  // single line
+              TRUE   // link from index
+              );
+          ol.endMemberDescription();
+        }
+        ol.endMemberDeclaration(0,0);
       }
-      else
-      {
-        ol.startBold();
-        ol.docify(fd->name()); 
-        ol.endBold();
-      }
-      if (fd->generateSourceFile())
-      {
-        ol.pushGeneratorState();
-        ol.disableAllBut(OutputGenerator::Html);
-        ol.docify(" ");
-        ol.startTextLink(fd->includeName(),0);
-        ol.docify("[");
-        ol.parseText(theTranslator->trCode());
-        ol.docify("]");
-        ol.endTextLink();
-        ol.popGeneratorState();
-      }
-      ol.endMemberItem();
-      if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
-      {
-        ol.startMemberDescription(fd->getOutputFileBase());
-        ol.generateDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
-            FALSE, // indexWords
-            FALSE, // isExample
-            0,     // exampleName
-            TRUE,  // single line
-            TRUE   // link from index
-           );
-        ol.endMemberDescription();
-      }
-      ol.endMemberDeclaration(0,0);
     }
     ol.endMemberList();
   }
@@ -329,7 +364,7 @@ QCString DirDef::shortTitle() const
 
 bool DirDef::hasDetailedDescription() const
 {
-  static bool repeatBrief = Config_getBool("REPEAT_BRIEF");
+  static bool repeatBrief = Config_getBool(REPEAT_BRIEF);
   return (!briefDescription().isEmpty() && repeatBrief) || !documentation().isEmpty();
 }
 
@@ -382,11 +417,11 @@ void DirDef::writeTagFile(FTextStream &tagFile)
 
 void DirDef::writeDocumentation(OutputList &ol)
 {
-  static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
+  static bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
   
   QCString title=theTranslator->trDirReference(m_dispName);
-  startFile(ol,getOutputFileBase(),name(),title,HLI_None,!generateTreeView);
+  startFile(ol,getOutputFileBase(),name(),title,HLI_Files,!generateTreeView);
 
   if (!generateTreeView)
   {
@@ -450,8 +485,14 @@ void DirDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::NamespaceNestedNamespaces:
       case LayoutDocEntry::NamespaceNestedConstantGroups:
       case LayoutDocEntry::NamespaceClasses:
+      case LayoutDocEntry::NamespaceInterfaces:
+      case LayoutDocEntry::NamespaceStructs:
+      case LayoutDocEntry::NamespaceExceptions:
       case LayoutDocEntry::NamespaceInlineClasses:
       case LayoutDocEntry::FileClasses:
+      case LayoutDocEntry::FileInterfaces:
+      case LayoutDocEntry::FileStructs:
+      case LayoutDocEntry::FileExceptions:
       case LayoutDocEntry::FileNamespaces:
       case LayoutDocEntry::FileConstantGroups:
       case LayoutDocEntry::FileIncludes:
@@ -592,11 +633,20 @@ void DirDef::computeDependencies()
               //printf("      %d: add dependency %s->%s\n",count++,name().data(),usedDir->name().data());
               addUsesDependency(usedDir,fd,ii->fileDef,FALSE);
             }
-          } 
+          }
         }
       }
     }
   }
+  if (m_usedDirs)
+  {
+    QDictIterator<UsedDir> udi(*m_usedDirs);
+    UsedDir *udir;
+    for (udi.toFirst();(udir=udi.current());++udi)
+    {
+      udir->sort();
+    }
+  }
 }
 
 bool DirDef::isParentOf(DirDef *dir) const
@@ -611,7 +661,7 @@ bool DirDef::isParentOf(DirDef *dir) const
 
 bool DirDef::depGraphIsTrivial() const
 {
-  return FALSE;
+  return m_usedDirs->count()==0;
 }
 
 //----------------------------------------------------------------------
@@ -619,8 +669,9 @@ bool DirDef::depGraphIsTrivial() const
 int FilePairDict::compareValues(const FilePair *left,const FilePair *right) const
 {
   int orderHi = qstricmp(left->source()->name(),right->source()->name());
+  if (orderHi!=0) return orderHi;
   int orderLo = qstricmp(left->destination()->name(),right->destination()->name());
-  return orderHi==0 ? orderLo : orderHi;
+  return orderLo;
 }
 
 //----------------------------------------------------------------------
@@ -638,10 +689,15 @@ UsedDir::~UsedDir()
 
 void UsedDir::addFileDep(FileDef *srcFd,FileDef *dstFd)
 {
-  m_filePairs.inSort(srcFd->getOutputFileBase()+dstFd->getOutputFileBase(),
+  m_filePairs.append(srcFd->getOutputFileBase()+dstFd->getOutputFileBase(),
                      new FilePair(srcFd,dstFd));
 }
 
+void UsedDir::sort()
+{
+  m_filePairs.sort();
+}
+
 FilePair *UsedDir::findFilePair(const char *name)
 {
   QCString n=name;
@@ -657,7 +713,7 @@ DirDef *DirDef::createNewDir(const char *path)
     //printf("Adding new dir %s\n",path);
     dir = new DirDef(path);
     //printf("createNewDir %s short=%s\n",path,dir->shortName().data());
-    Doxygen::directories->inSort(path,dir);
+    Doxygen::directories->append(path,dir);
   }
   return dir;
 }
@@ -678,7 +734,7 @@ bool DirDef::matchPath(const QCString &path,QStrList &l)
 }
 
 /*! strip part of \a path if it matches
- *  one of the paths in the Config_getList("STRIP_FROM_PATH") list
+ *  one of the paths in the Config_getList(STRIP_FROM_PATH) list
  */
 DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
 {
@@ -688,7 +744,7 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
   while ((i=path.find('/',p))!=-1)
   {
     QCString part=path.left(i+1);
-    if (!matchPath(part,Config_getList("STRIP_FROM_PATH")) && (part!="/" && part!="//"))
+    if (!matchPath(part,Config_getList(STRIP_FROM_PATH)) && (part!="/" && part!="//"))
     {
       dir=createNewDir(part); 
     }
@@ -697,11 +753,6 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
   return dir;
 }
 
-void DirDef::writeDepGraph(FTextStream &t)
-{
-    writeDotDirDepGraph(t,this);
-}
-
 //----------------------------------------------------------------------
 
 static void writePartialDirPath(OutputList &ol,const DirDef *root,const DirDef *target)
@@ -735,7 +786,7 @@ static void writePartialFilePath(OutputList &ol,const DirDef *root,const FileDef
 
 void DirRelation::writeDocumentation(OutputList &ol)
 {
-  static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
+  static bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
   ol.disableAllBut(OutputGenerator::Html);
 
@@ -842,6 +893,7 @@ static void computeCommonDirPrefix()
           else // dir is shorter than path -> take path of dir as new start
           {
             path=dir->name();
+            l=path.length();
             int i=path.findRev('/',l-2);
             if (i==-1) // no unique prefix -> stop
             {
@@ -884,14 +936,14 @@ void buildDirectories()
     for (;(fd=fni.current());++fni)
     {
       //printf("buildDirectories %s\n",fd->name().data());
-      if (fd->getReference().isEmpty() && !fd->isDocumentationFile())
+      if (fd->getReference().isEmpty())
       {
         DirDef *dir;
         if ((dir=Doxygen::directories->find(fd->getPath()))==0) // new directory
         {
           dir = DirDef::mergeDirectoryInTree(fd->getPath());
         }
-        if (dir) dir->addFile(fd);
+        if (dir && !fd->isDocumentationFile()) dir->addFile(fd);
       }
       else
       {
@@ -906,7 +958,6 @@ void buildDirectories()
   DirSDict::Iterator sdi(*Doxygen::directories);
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
-    //printf("New dir %s\n",dir->displayName().data());
     QCString name = dir->name();
     int i=name.findRev('/',name.length()-2);
     if (i>0)
@@ -921,6 +972,11 @@ void buildDirectories()
       }
     }
   }
+  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  {
+    dir->sort();
+  }
+  Doxygen::directories->sort();
   computeCommonDirPrefix();
 }
 
@@ -948,9 +1004,15 @@ void generateDirDocs(OutputList &ol)
   DirSDict::Iterator sdi(*Doxygen::directories);
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
+    ol.pushGeneratorState();
+    if (!dir->hasDocumentation())
+    {
+      ol.disableAllBut(OutputGenerator::Html);
+    }
     dir->writeDocumentation(ol);
+    ol.popGeneratorState();
   }
-  if (Config_getBool("DIRECTORY_GRAPH"))
+  if (Config_getBool(DIRECTORY_GRAPH))
   {
     SDict<DirRelation>::Iterator rdi(Doxygen::dirRelations);
     DirRelation *dr;
