@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,6 +21,7 @@
 
 #include <qcstring.h>
 #include <qlist.h>
+#include <stdio.h>
 #include "htmlattrib.h"
 
 class Definition;
@@ -33,12 +34,13 @@ enum Tokens
   TK_WHITESPACE    = 3,
   TK_LISTITEM      = 4,
   TK_ENDLIST       = 5,
-  TK_COMMAND       = 6,
+  TK_COMMAND_AT    = 6, //! Command starting with `@`
   TK_HTMLTAG       = 7,
   TK_SYMBOL        = 8,
   TK_NEWPARA       = 9,
   TK_RCSTAG        = 10,
   TK_URL           = 11,
+  TK_COMMAND_BS    = 12, //! Command starting with `\`
 
   RetVal_OK             = 0x10000,
   RetVal_SimpleSec      = 0x10001,
@@ -61,7 +63,9 @@ enum Tokens
   RetVal_SwitchLang     = 0x10012,
   RetVal_CloseXml       = 0x10013,
   RetVal_EndBlockQuote  = 0x10014,
-  RetVal_CopyDoc        = 0x10015
+  RetVal_CopyDoc        = 0x10015,
+  RetVal_EndInternal    = 0x10016,
+  RetVal_EndParBlock    = 0x10017
 };
 
 /** @brief Data associated with a token used by the comment block parser. */
@@ -136,6 +140,7 @@ void doctokenizerYYsetStateHtmlOnly();
 void doctokenizerYYsetStateManOnly();
 void doctokenizerYYsetStateLatexOnly();
 void doctokenizerYYsetStateXmlOnly();
+void doctokenizerYYsetStateDbOnly();
 void doctokenizerYYsetStateRtfOnly();
 void doctokenizerYYsetStateVerbatim();
 void doctokenizerYYsetStateDot();
@@ -154,5 +159,13 @@ void doctokenizerYYsetStateAnchor();
 void doctokenizerYYsetInsidePre(bool b);
 void doctokenizerYYpushBackHtmlTag(const char *tag);
 void doctokenizerYYsetStateSnippet();
+void doctokenizerYYstartAutoList();
+void doctokenizerYYendAutoList();
+void doctokenizerYYsetStatePlantUML();
+void doctokenizerYYsetStateSetScope();
+void doctokenizerYYsetStatePlantUMLOpt();
+void doctokenizerYYsetStateOptions();
+void doctokenizerYYsetStateBlock();
+void doctokenizerYYsetStateEmoji();
 
 #endif
