@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -82,10 +82,12 @@ FileDef::FileDef(const char *p,const char *nm,
   m_path=p;
   m_filePath=m_path+nm;
   m_fileName=nm;
-  m_diskName=dn;
-  if (m_diskName.isEmpty()) m_diskName=nm;
   setReference(lref);
+  setDiskName(dn?dn:nm);
   m_classSDict        = 0;
+  m_interfaceSDict    = 0;
+  m_structSDict       = 0;
+  m_exceptionSDict    = 0;
   m_includeList       = 0;
   m_includeDict       = 0; 
   m_includedByList    = 0;
@@ -99,20 +101,23 @@ FileDef::FileDef(const char *p,const char *nm,
   m_isSource          = guessSection(nm)==Entry::SOURCE_SEC; 
   m_docname           = nm;
   m_dir               = 0;
-  if (Config_getBool("FULL_PATH_NAMES"))
+  if (Config_getBool(FULL_PATH_NAMES))
   {
     m_docname.prepend(stripFromPath(m_path.copy()));
   }
   setLanguage(getLanguageFromFileName(name()));
   m_memberGroupSDict = 0;
   acquireFileVersion();
-  m_subGrouping=Config_getBool("SUBGROUPING");
+  m_subGrouping=Config_getBool(SUBGROUPING);
 }
 
 /*! destroy the file definition */
 FileDef::~FileDef()
 {
   delete m_classSDict;
+  delete m_interfaceSDict;
+  delete m_structSDict;
+  delete m_exceptionSDict;
   delete m_includeDict;
   delete m_includeList;
   delete m_includedByDict;
@@ -125,6 +130,22 @@ FileDef::~FileDef()
   delete m_memberGroupSDict;
 }
 
+void FileDef::setDiskName(const QCString &name)
+{
+  if (isReference())
+  {
+    m_outputDiskName = name;
+    m_inclDepFileName = name+"_incl";
+    m_inclByDepFileName = name+"_dep_incl";
+  }
+  else
+  {
+    m_outputDiskName = convertNameToFile(name);
+    m_inclDepFileName = convertNameToFile(name+"_incl");
+    m_inclByDepFileName = convertNameToFile(name+"_dep_incl");
+  }
+}
+
 /*! Compute the HTML anchor names for all members in the class */ 
 void FileDef::computeAnchors()
 {
@@ -172,8 +193,8 @@ void FileDef::findSectionsInDocumentation()
 
 bool FileDef::hasDetailedDescription() const
 {
-  static bool repeatBrief = Config_getBool("REPEAT_BRIEF");
-  static bool sourceBrowser = Config_getBool("SOURCE_BROWSER");
+  static bool repeatBrief = Config_getBool(REPEAT_BRIEF);
+  static bool sourceBrowser = Config_getBool(SOURCE_BROWSER);
   return ((!briefDescription().isEmpty() && repeatBrief) || 
           !documentation().stripWhiteSpace().isEmpty() || // avail empty section
           (sourceBrowser && getStartBodyLine()!=-1 && getBodyDef())
@@ -224,18 +245,25 @@ void FileDef::writeTagFile(FTextStream &tagFile)
       case LayoutDocEntry::FileClasses:
         {
           if (m_classSDict)
-          {
-            SDict<ClassDef>::Iterator ci(*m_classSDict);
-            ClassDef *cd;
-            for (ci.toFirst();(cd=ci.current());++ci)
-            {
-              if (cd->isLinkableInProject())
-              {
-                tagFile << "    <class kind=\"" << cd->compoundTypeString() <<
-                  "\">" << convertToXML(cd->name()) << "</class>" << endl;
-              }
-            }
-          }
+            writeClassesToTagFile(tagFile, m_classSDict);
+        }
+        break;
+      case LayoutDocEntry::FileInterfaces:
+        {
+          if (m_interfaceSDict)
+            writeClassesToTagFile(tagFile, m_interfaceSDict);
+        }
+        break;
+      case LayoutDocEntry::FileStructs:
+        {
+          if (m_structSDict)
+            writeClassesToTagFile(tagFile, m_structSDict);
+        }
+        break;
+      case LayoutDocEntry::FileExceptions:
+        {
+          if (m_exceptionSDict)
+            writeClassesToTagFile(tagFile, m_exceptionSDict);
         }
         break;
       case LayoutDocEntry::FileNamespaces:
@@ -303,11 +331,11 @@ void FileDef::writeDetailedDescription(OutputList &ol,const QCString &title)
     ol.endGroupHeader();
 
     ol.startTextBlock();
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
+    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
       ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
     }
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
+    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) && 
         !documentation().isEmpty())
     {
       ol.pushGeneratorState();
@@ -325,20 +353,24 @@ void FileDef::writeDetailedDescription(OutputList &ol,const QCString &title)
       ol.generateDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
     }
     //printf("Writing source ref for file %s\n",name().data());
-    if (Config_getBool("SOURCE_BROWSER")) 
+    if (Config_getBool(SOURCE_BROWSER)) 
     {
       //if Latex enabled and LATEX_SOURCE_CODE isn't -> skip, bug_738548
       ol.pushGeneratorState();
-      if (ol.isEnabled(OutputGenerator::Latex) && !Config_getBool("LATEX_SOURCE_CODE"))
+      if (ol.isEnabled(OutputGenerator::Latex) && !Config_getBool(LATEX_SOURCE_CODE))
       { 
         ol.disable(OutputGenerator::Latex);
       }
-      if (ol.isEnabled(OutputGenerator::RTF) && !Config_getBool("RTF_SOURCE_CODE"))
+      if (ol.isEnabled(OutputGenerator::Docbook) && !Config_getBool(DOCBOOK_PROGRAMLISTING))
+      { 
+        ol.disable(OutputGenerator::Docbook);
+      }
+      if (ol.isEnabled(OutputGenerator::RTF) && !Config_getBool(RTF_SOURCE_CODE))
       { 
         ol.disable(OutputGenerator::RTF);
       }
 
-      ol.startParagraph();
+      ol.startParagraph("definition");
       QCString refText = theTranslator->trDefinedInSourceFile();
       int fileMarkerPos = refText.find("@0");
       if (fileMarkerPos!=-1) // should always pass this.
@@ -349,6 +381,10 @@ void FileDef::writeDetailedDescription(OutputList &ol,const QCString &title)
         ol.parseText(refText.right(
               refText.length()-fileMarkerPos-2)); // text right from marker 2
       }
+      else
+      {
+        err("translation error: invalid marker in trDefinedInSourceFile()\n");
+      }
       ol.endParagraph();
       //Restore settings, bug_738548
       ol.popGeneratorState();
@@ -359,7 +395,7 @@ void FileDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 
 void FileDef::writeBriefDescription(OutputList &ol)
 {
-  if (!briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
+  if (hasBriefDescription())
   {
     DocRoot *rootNode = validatingParseDoc(briefFile(),briefLine(),this,0,
                        briefDescription(),TRUE,FALSE,0,TRUE,FALSE);
@@ -367,13 +403,17 @@ void FileDef::writeBriefDescription(OutputList &ol)
     if (rootNode && !rootNode->isEmpty())
     {
       ol.startParagraph();
+      ol.pushGeneratorState();
+      ol.disableAllBut(OutputGenerator::Man);
+      ol.writeString(" - ");
+      ol.popGeneratorState();
       ol.writeDoc(rootNode,this,0);
       ol.pushGeneratorState();
       ol.disable(OutputGenerator::RTF);
       ol.writeString(" \n");
       ol.enable(OutputGenerator::RTF);
 
-      if (Config_getBool("REPEAT_BRIEF") ||
+      if (Config_getBool(REPEAT_BRIEF) ||
           !documentation().isEmpty()
          )
       {
@@ -390,6 +430,20 @@ void FileDef::writeBriefDescription(OutputList &ol)
   ol.writeSynopsis();
 }
 
+void FileDef::writeClassesToTagFile(FTextStream &tagFile, ClassSDict *d)
+{
+  SDict<ClassDef>::Iterator ci(*d);
+  ClassDef *cd;
+  for (ci.toFirst();(cd=ci.current());++ci)
+  {
+    if (cd->isLinkableInProject())
+    {
+      tagFile << "    <class kind=\"" << cd->compoundTypeString() <<
+        "\">" << convertToXML(cd->name()) << "</class>" << endl;
+    }
+  }
+}
+
 void FileDef::writeIncludeFiles(OutputList &ol)
 {
   if (m_includeList && m_includeList->count()>0)
@@ -461,7 +515,7 @@ void FileDef::writeIncludeFiles(OutputList &ol)
 
 void FileDef::writeIncludeGraph(OutputList &ol)
 {
-  if (Config_getBool("HAVE_DOT") /*&& Config_getBool("INCLUDE_GRAPH")*/)
+  if (Config_getBool(HAVE_DOT) /*&& Config_getBool(INCLUDE_GRAPH)*/)
   {
     //printf("Graph for file %s\n",name().data());
     DotInclDepGraph incDepGraph(this,FALSE);
@@ -479,13 +533,13 @@ void FileDef::writeIncludeGraph(OutputList &ol)
       ol.enableAll();
       ol.endTextBlock(TRUE);
     }
-    //incDepGraph.writeGraph(Config_getString("HTML_OUTPUT"),fd->getOutputFileBase());
+    //incDepGraph.writeGraph(Config_getString(HTML_OUTPUT),fd->getOutputFileBase());
   }
 }
 
 void FileDef::writeIncludedByGraph(OutputList &ol)
 {
-  if (Config_getBool("HAVE_DOT") /*&& Config_getBool("INCLUDED_BY_GRAPH")*/)
+  if (Config_getBool(HAVE_DOT) /*&& Config_getBool(INCLUDED_BY_GRAPH)*/)
   {
     //printf("Graph for file %s\n",name().data());
     DotInclDepGraph incDepGraph(this,TRUE);
@@ -503,7 +557,7 @@ void FileDef::writeIncludedByGraph(OutputList &ol)
       ol.enableAll();
       ol.endTextBlock(TRUE);
     }
-    //incDepGraph.writeGraph(Config_getString("HTML_OUTPUT"),fd->getOutputFileBase());
+    //incDepGraph.writeGraph(Config_getString(HTML_OUTPUT),fd->getOutputFileBase());
   }
 }
 
@@ -530,10 +584,10 @@ void FileDef::writeNamespaceDeclarations(OutputList &ol,const QCString &title,
   if (m_namespaceSDict) m_namespaceSDict->writeDeclaration(ol,title,isConstantGroup);
 }
 
-void FileDef::writeClassDeclarations(OutputList &ol,const QCString &title)
+void FileDef::writeClassDeclarations(OutputList &ol,const QCString &title,ClassSDict *d)
 {
   // write list of classes
-  if (m_classSDict) m_classSDict->writeDeclaration(ol,0,title,FALSE);
+  if (d) d->writeDeclaration(ol,0,title,FALSE);
 }
 
 void FileDef::writeInlineClasses(OutputList &ol)
@@ -561,7 +615,7 @@ void FileDef::endMemberDeclarations(OutputList &ol)
 
 void FileDef::startMemberDocumentation(OutputList &ol)
 {
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  if (Config_getBool(SEPARATE_MEMBER_PAGES))
   {
     ol.disable(OutputGenerator::Html);
     Doxygen::suppressDocWarnings = TRUE;
@@ -570,7 +624,7 @@ void FileDef::startMemberDocumentation(OutputList &ol)
 
 void FileDef::endMemberDocumentation(OutputList &ol)
 {
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  if (Config_getBool(SEPARATE_MEMBER_PAGES))
   {
     ol.enable(OutputGenerator::Html);
     Doxygen::suppressDocWarnings = FALSE;
@@ -604,7 +658,7 @@ void FileDef::writeAuthorSection(OutputList &ol)
   ol.startGroupHeader();
   ol.parseText(theTranslator->trAuthor(TRUE,TRUE));
   ol.endGroupHeader();
-  ol.parseText(theTranslator->trGeneratedAutomatically(Config_getString("PROJECT_NAME")));
+  ol.parseText(theTranslator->trGeneratedAutomatically(Config_getString(PROJECT_NAME)));
   ol.popGeneratorState();
 }
 
@@ -619,14 +673,38 @@ void FileDef::writeSummaryLinks(OutputList &ol)
   SrcLangExt lang=getLanguage();
   for (eli.toFirst();(lde=eli.current());++eli)
   {
-    if ((lde->kind()==LayoutDocEntry::FileClasses && 
-         m_classSDict && m_classSDict->declVisible()) || 
-        (lde->kind()==LayoutDocEntry::FileNamespaces && 
-         m_namespaceSDict && m_namespaceSDict->declVisible())
-       )
+    if (lde->kind()==LayoutDocEntry::FileClasses && m_classSDict && m_classSDict->declVisible())
     {
       LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-      QCString label = lde->kind()==LayoutDocEntry::FileClasses ? "nested-classes" : "namespaces";
+      QCString label = "nested-classes";
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
+    }
+    else if (lde->kind()==LayoutDocEntry::FileInterfaces && m_interfaceSDict && m_interfaceSDict->declVisible())
+    {
+      LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+      QCString label = "interfaces";
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
+    }
+    else if (lde->kind()==LayoutDocEntry::FileStructs && m_structSDict && m_structSDict->declVisible())
+    {
+      LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+      QCString label = "structs";
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
+    }
+    else if (lde->kind()==LayoutDocEntry::FileExceptions && m_exceptionSDict && m_exceptionSDict->declVisible())
+    {
+      LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+      QCString label = "exceptions";
+      ol.writeSummaryLink(0,label,ls->title(lang),first);
+      first=FALSE;
+    }
+    else if (lde->kind()==LayoutDocEntry::FileNamespaces && m_namespaceSDict && m_namespaceSDict->declVisible())
+    {
+      LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+      QCString label = "namespaces";
       ol.writeSummaryLink(0,label,ls->title(lang),first);
       first=FALSE;
     }
@@ -653,11 +731,11 @@ void FileDef::writeSummaryLinks(OutputList &ol)
 */
 void FileDef::writeDocumentation(OutputList &ol)
 {
-  static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
+  static bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   //funcList->countDecMembers();
   
   //QCString fn = name();
-  //if (Config_getBool("FULL_PATH_NAMES"))
+  //if (Config_getBool(FULL_PATH_NAMES))
   //{
   //  fn.prepend(stripFromPath(getPath().copy()));
   //}
@@ -754,9 +832,27 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::FileClasses: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
-          writeClassDeclarations(ol,ls->title(lang));
+          writeClassDeclarations(ol,ls->title(lang),m_classSDict);
         }
-        break; 
+        break;
+      case LayoutDocEntry::FileInterfaces: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeClassDeclarations(ol,ls->title(lang),m_interfaceSDict);
+        }
+        break;
+      case LayoutDocEntry::FileStructs: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeClassDeclarations(ol,ls->title(lang),m_structSDict);
+        }
+        break;
+      case LayoutDocEntry::FileExceptions: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeClassDeclarations(ol,ls->title(lang),m_exceptionSDict);
+        }
+        break;
       case LayoutDocEntry::FileNamespaces: 
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
@@ -815,6 +911,9 @@ void FileDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::NamespaceNestedNamespaces:
       case LayoutDocEntry::NamespaceNestedConstantGroups:
       case LayoutDocEntry::NamespaceClasses:
+      case LayoutDocEntry::NamespaceInterfaces:
+      case LayoutDocEntry::NamespaceStructs:
+      case LayoutDocEntry::NamespaceExceptions:
       case LayoutDocEntry::NamespaceInlineClasses:
       case LayoutDocEntry::GroupClasses: 
       case LayoutDocEntry::GroupInlineClasses: 
@@ -839,7 +938,7 @@ void FileDef::writeDocumentation(OutputList &ol)
 
   endFileWithNavPath(this,ol);
 
-  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  if (Config_getBool(SEPARATE_MEMBER_PAGES))
   {
     MemberList *ml = getMemberList(MemberListType_allMembersList);
     if (ml) ml->sort();
@@ -867,7 +966,7 @@ void FileDef::writeMemberPages(OutputList &ol)
 
 void FileDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
 {
-  static bool createSubDirs=Config_getBool("CREATE_SUBDIRS");
+  static bool createSubDirs=Config_getBool(CREATE_SUBDIRS);
 
   ol.writeString("      <div class=\"navtab\">\n");
   ol.writeString("        <table>\n");
@@ -911,10 +1010,11 @@ void FileDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
 /*! Write a source listing of this file to the output */
 void FileDef::writeSource(OutputList &ol,bool sameTu,QStrList &filesInSameTu)
 {
-  static bool generateTreeView  = Config_getBool("GENERATE_TREEVIEW");
-  static bool filterSourceFiles = Config_getBool("FILTER_SOURCE_FILES");
-  static bool latexSourceCode   = Config_getBool("LATEX_SOURCE_CODE");
-  static bool rtfSourceCode     = Config_getBool("RTF_SOURCE_CODE");
+  static bool generateTreeView  = Config_getBool(GENERATE_TREEVIEW);
+  static bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
+  static bool latexSourceCode   = Config_getBool(LATEX_SOURCE_CODE);
+  static bool docbookSourceCode = Config_getBool(DOCBOOK_PROGRAMLISTING);
+  static bool rtfSourceCode     = Config_getBool(RTF_SOURCE_CODE);
   DevNullCodeDocInterface devNullIntf;
   QCString title = m_docname;
   if (!m_fileVersion.isEmpty())
@@ -924,6 +1024,7 @@ void FileDef::writeSource(OutputList &ol,bool sameTu,QStrList &filesInSameTu)
   QCString pageTitle = theTranslator->trSourceFile(title);
   ol.disable(OutputGenerator::Man);
   if (!latexSourceCode) ol.disable(OutputGenerator::Latex);
+  if (!docbookSourceCode) ol.disable(OutputGenerator::Docbook);
   if (!rtfSourceCode) ol.disable(OutputGenerator::RTF);
 
   bool isDocFile = isDocumentationFile();
@@ -955,19 +1056,20 @@ void FileDef::writeSource(OutputList &ol,bool sameTu,QStrList &filesInSameTu)
 
   if (isLinkable())
   {
+    ol.pushGeneratorState();
     if (latexSourceCode) ol.disable(OutputGenerator::Latex);
     if (rtfSourceCode) ol.disable(OutputGenerator::RTF);
+    if (docbookSourceCode) ol.disable(OutputGenerator::Docbook);
     ol.startTextLink(getOutputFileBase(),0);
     ol.parseText(theTranslator->trGotoDocumentation());
     ol.endTextLink();
-    if (latexSourceCode) ol.enable(OutputGenerator::Latex);
-    if (rtfSourceCode) ol.enable(OutputGenerator::RTF);
+    ol.popGeneratorState();
   }
 
   (void)sameTu;
   (void)filesInSameTu;
 #if USE_LIBCLANG
-  static bool clangAssistedParsing = Config_getBool("CLANG_ASSISTED_PARSING");
+  static bool clangAssistedParsing = Config_getBool(CLANG_ASSISTED_PARSING);
   if (clangAssistedParsing && 
       (getLanguage()==SrcLangExt_Cpp || getLanguage()==SrcLangExt_ObjC))
   {
@@ -1026,12 +1128,12 @@ void FileDef::writeSource(OutputList &ol,bool sameTu,QStrList &filesInSameTu)
 
 void FileDef::parseSource(bool sameTu,QStrList &filesInSameTu)
 {
-  static bool filterSourceFiles = Config_getBool("FILTER_SOURCE_FILES");
+  static bool filterSourceFiles = Config_getBool(FILTER_SOURCE_FILES);
   DevNullCodeDocInterface devNullIntf;
   (void)sameTu;
   (void)filesInSameTu;
 #if USE_LIBCLANG
-  static bool clangAssistedParsing = Config_getBool("CLANG_ASSISTED_PARSING");
+  static bool clangAssistedParsing = Config_getBool(CLANG_ASSISTED_PARSING);
   if (clangAssistedParsing && 
       (getLanguage()==SrcLangExt_Cpp || getLanguage()==SrcLangExt_ObjC))
   {
@@ -1130,6 +1232,14 @@ void FileDef::insertMember(MemberDef *md)
       addMemberToList(MemberListType_decTypedefMembers,md);
       addMemberToList(MemberListType_docTypedefMembers,md);
       break;
+    case MemberType_Sequence:      
+      addMemberToList(MemberListType_decSequenceMembers,md);
+      addMemberToList(MemberListType_docSequenceMembers,md);
+      break;
+    case MemberType_Dictionary:      
+      addMemberToList(MemberListType_decDictionaryMembers,md);
+      addMemberToList(MemberListType_docDictionaryMembers,md);
+      break;
     case MemberType_Enumeration:  
       addMemberToList(MemberListType_decEnumMembers,md);
       addMemberToList(MemberListType_docEnumMembers,md);
@@ -1154,17 +1264,36 @@ void FileDef::insertMember(MemberDef *md)
 void FileDef::insertClass(ClassDef *cd)
 {
   if (cd->isHidden()) return;
-  if (m_classSDict==0)
+
+  ClassSDict *d=0;
+  ClassSDict **dd=&m_classSDict;
+
+  if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
   {
-    m_classSDict = new ClassSDict(17);
+    if (cd->compoundType()==ClassDef::Interface)
+    {
+      dd = &m_interfaceSDict;
+    }
+    else if (cd->compoundType()==ClassDef::Struct)
+    {
+      dd = &m_structSDict;
+    }
+    else if (cd->compoundType()==ClassDef::Exception)
+    {
+      dd = &m_exceptionSDict;
+    }
   }
-  if (Config_getBool("SORT_BRIEF_DOCS"))
+
+  if (*dd==0) *dd = new ClassSDict(17);
+  d = *dd;
+
+  if (Config_getBool(SORT_BRIEF_DOCS))
   {
-    m_classSDict->inSort(cd->name(),cd);
+    d->inSort(cd->name(),cd);
   }
   else
   {
-    m_classSDict->append(cd->name(),cd);
+    d->append(cd->name(),cd);
   }
 }
 
@@ -1179,7 +1308,7 @@ void FileDef::insertNamespace(NamespaceDef *nd)
     {
       m_namespaceSDict = new NamespaceSDict;
     }
-    if (Config_getBool("SORT_BRIEF_DOCS"))
+    if (Config_getBool(SORT_BRIEF_DOCS))
     {
       m_namespaceSDict->inSort(nd->name(),nd);
     }
@@ -1192,7 +1321,7 @@ void FileDef::insertNamespace(NamespaceDef *nd)
 
 QCString FileDef::name() const 
 { 
-  if (Config_getBool("FULL_PATH_NAMES")) 
+  if (Config_getBool(FULL_PATH_NAMES)) 
     return m_fileName; 
   else 
     return Definition::name(); 
@@ -1395,15 +1524,13 @@ bool FileDef::isIncluded(const QCString &name) const
 
 bool FileDef::generateSourceFile() const 
 { 
-  static bool sourceBrowser = Config_getBool("SOURCE_BROWSER");
-  static bool verbatimHeaders = Config_getBool("VERBATIM_HEADERS");
-  QCString extension = name().right(4);
+  static bool sourceBrowser = Config_getBool(SOURCE_BROWSER);
+  static bool verbatimHeaders = Config_getBool(VERBATIM_HEADERS);
   return !isReference() && 
          (sourceBrowser || 
            (verbatimHeaders && guessSection(name())==Entry::HEADER_SEC) 
          ) &&
-         extension!=".doc" && extension!=".txt" && extension!=".dox" &&
-         extension!=".md" && name().right(9)!=".markdown";  
+         !isDocumentationFile();
 }
 
 
@@ -1736,12 +1863,13 @@ bool FileDef::isDocumentationFile() const
 
 void FileDef::acquireFileVersion()
 {
-  QCString vercmd = Config_getString("FILE_VERSION_FILTER");
-  if (!vercmd.isEmpty() && !m_filePath.isEmpty() && m_filePath!="generated") 
+  QCString vercmd = Config_getString(FILE_VERSION_FILTER);
+  if (!vercmd.isEmpty() && !m_filePath.isEmpty() &&
+      m_filePath!="generated" && m_filePath!="graph_legend")
   {
     msg("Version of %s : ",m_filePath.data());
     QCString cmd = vercmd+" \""+m_filePath+"\"";
-    Debug::print(Debug::ExtCmd,0,"Executing popen(`%s`)\n",cmd.data());
+    Debug::print(Debug::ExtCmd,0,"Executing popen(`%s`)\n",qPrint(cmd));
     FILE *f=portable_popen(cmd,"r");
     if (!f)
     {
@@ -1768,17 +1896,22 @@ void FileDef::acquireFileVersion()
 
 
 QCString FileDef::getSourceFileBase() const
-{ 
+{
   if (Htags::useHtags)
   {
     return Htags::path2URL(m_filePath);
   }
   else
   {
-    return convertNameToFile(m_diskName)+"_source"; 
+    return m_outputDiskName+"_source";
   }
 }
 
+QCString FileDef::getOutputFileBase() const
+{
+  return m_outputDiskName;
+}
+
 /*! Returns the name of the verbatim copy of this file (if any). */
 QCString FileDef::includeName() const 
 { 
@@ -1805,8 +1938,8 @@ MemberList *FileDef::createMemberList(MemberListType lt)
 
 void FileDef::addMemberToList(MemberListType lt,MemberDef *md)
 {
-  static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
-  static bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
+  static bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
+  static bool sortMemberDocs = Config_getBool(SORT_MEMBER_DOCS);
   MemberList *ml = createMemberList(lt);
   ml->setNeedsSorting(
        ((ml->listType()&MemberListType_declarationLists) && sortBriefDocs) ||
@@ -1851,7 +1984,7 @@ MemberList *FileDef::getMemberList(MemberListType lt) const
 
 void FileDef::writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title)
 {
-  static bool optVhdl = Config_getBool("OPTIMIZE_OUTPUT_VHDL");
+  static bool optVhdl = Config_getBool(OPTIMIZE_OUTPUT_VHDL);
   MemberList * ml = getMemberList(lt);
   if (ml) 
   {
@@ -1875,8 +2008,8 @@ void FileDef::writeMemberDocumentation(OutputList &ol,MemberListType lt,const QC
 
 bool FileDef::isLinkableInProject() const
 {
-  static bool showFiles = Config_getBool("SHOW_FILES");
-  return hasDocumentation() && !isReference() && showFiles;
+  static bool showFiles = Config_getBool(SHOW_FILES);
+  return hasDocumentation() && !isReference() && (showFiles || isLinkableViaGroup());
 }
 
 static void getAllIncludeFilesRecursively(
@@ -1915,3 +2048,14 @@ QCString FileDef::fileVersion() const
 {
   return m_fileVersion;
 }
+
+QCString FileDef::includeDependencyGraphFileName() const
+{
+  return m_inclDepFileName;
+}
+
+QCString FileDef::includedByDependencyGraphFileName() const
+{
+  return m_inclByDepFileName;
+}
+
