@@ -1,8 +1,7 @@
 /******************************************************************************
- i
- * $Id: formula.cpp,v 1.16 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -16,15 +15,9 @@
  */
 
 #include <stdlib.h>
-#include <unistd.h>
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
-#include "qtbc.h"
 #include <qfile.h>
-#include <qtextstream.h>
 #include <qfileinfo.h>
+#include <qtextstream.h>
 #include <qdir.h>
 
 #include "formula.h"
@@ -32,6 +25,10 @@
 #include "util.h"
 #include "message.h"
 #include "config.h"
+#include "portable.h"
+#include "index.h"
+#include "doxygen.h"
+#include "ftextstream.h"
 
 Formula::Formula(const char *text)
 {
@@ -54,8 +51,8 @@ void FormulaList::generateBitmaps(const char *path)
   int x1,y1,x2,y2;
   QDir d(path);
   // store the original directory
-  if (!d.exists()) { err("Error: Output dir %s does not exist!\n",path); exit(1); }
-  QCString oldDir = convertToQCString(QDir::currentDirPath());
+  if (!d.exists()) { err("Output dir %s does not exist!\n",path); exit(1); }
+  QCString oldDir = QDir::currentDirPath().utf8();
   // go to the html output directory (i.e. path)
   QDir::setCurrent(d.absPath());
   QDir thisDir;
@@ -69,16 +66,13 @@ void FormulaList::generateBitmaps(const char *path)
   bool formulaError=FALSE;
   if (f.open(IO_WriteOnly))
   {
-    QTextStream t(&f);
-    if (Config_getBool("LATEX_BATCHMODE")) t << "\\batchmode" << endl;
+    FTextStream t(&f);
+    if (Config_getBool(LATEX_BATCHMODE)) t << "\\batchmode" << endl;
     t << "\\documentclass{article}" << endl;
+    t << "\\usepackage{ifthen}" << endl;
     t << "\\usepackage{epsfig}" << endl; // for those who want to include images
-    const char *s=Config_getList("EXTRA_PACKAGES").first();
-    while (s)
-    {
-      t << "\\usepackage{" << s << "}\n";
-      s=Config_getList("EXTRA_PACKAGES").next();
-    }
+    writeExtraLatexPackages(t);
+    writeLatexSpecialFormulaChars(t);
     t << "\\pagestyle{empty}" << endl; 
     t << "\\begin{document}" << endl;
     int page=0;
@@ -94,6 +88,7 @@ void FormulaList::generateBitmaps(const char *path)
         t << formula->getFormulaText() << endl << "\\pagebreak\n\n";
         pagesToGenerate.append(new int(page));
       }
+      Doxygen::indexList->addImageFile(resultName);
       page++;
     }
     t << "\\end{document}" << endl;
@@ -103,12 +98,16 @@ void FormulaList::generateBitmaps(const char *path)
   {
     //printf("Running latex...\n");
     //system("latex _formulas.tex </dev/null >/dev/null");
-    if (iSystem("latex","_formulas.tex")!=0)
+    QCString latexCmd = "latex";
+    portable_sysTimerStart();
+    if (portable_system(latexCmd,"_formulas.tex")!=0)
     {
-      err("Problems running latex. Check your installation or look for typos in _formulas.tex!\n");
+      err("Problems running latex. Check your installation or look "
+          "for typos in _formulas.tex and check _formulas.log!\n");
       formulaError=TRUE;
       //return;
     }
+    portable_sysTimerStop();
     //printf("Running dvips...\n");
     QListIterator<int> pli(pagesToGenerate);
     int *pagePtr;
@@ -124,11 +123,15 @@ void FormulaList::generateBitmaps(const char *path)
       // encapsulated postscript.
       sprintf(dviArgs,"-q -D 600 -E -n 1 -p %d -o %s.eps _formulas.dvi",
           pageIndex,formBase.data());
-      if (iSystem("dvips",dviArgs)!=0)
+      portable_sysTimerStart();
+      if (portable_system("dvips",dviArgs)!=0)
       {
         err("Problems running dvips. Check your installation!\n");
+        portable_sysTimerStop();
+        QDir::setCurrent(oldDir);
         return;
       }
+      portable_sysTimerStop();
       // now we read the generated postscript file to extract the bounding box
       QFileInfo fi(formBase+".eps");
       if (fi.exists())
@@ -141,7 +144,7 @@ void FormulaList::generateBitmaps(const char *path)
         }
         else
         {
-          err("Error: Couldn't extract bounding box!\n");
+          err("Couldn't extract bounding box!\n");
         }
       } 
       // next we generate a postscript file which contains the eps
@@ -149,7 +152,7 @@ void FormulaList::generateBitmaps(const char *path)
       f.setName(formBase+".ps");
       if (f.open(IO_WriteOnly))
       {
-        QTextStream t(&f);
+        FTextStream t(&f);
         t << "1 1 1 setrgbcolor" << endl;  // anti-alias to white background
         t << "newpath" << endl;
         t << "-1 -1 moveto" << endl;
@@ -165,61 +168,31 @@ void FormulaList::generateBitmaps(const char *path)
       }
       // scale the image so that it is four times larger than needed.
       // and the sizes are a multiple of four.
-      const double scaleFactor = 16.0/3.0; 
-      int gx = (((int)((x2-x1)*scaleFactor))+3)&~2;
-      int gy = (((int)((y2-y1)*scaleFactor))+3)&~2;
+      double scaleFactor = 16.0/3.0; 
+      int zoomFactor = Config_getInt(FORMULA_FONTSIZE);
+      if (zoomFactor<8 || zoomFactor>50) zoomFactor=10;
+      scaleFactor *= zoomFactor/10.0;
+      int gx = (((int)((x2-x1)*scaleFactor))+3)&~1;
+      int gy = (((int)((y2-y1)*scaleFactor))+3)&~1;
       // Then we run ghostscript to convert the postscript to a pixmap
       // The pixmap is a truecolor image, where only black and white are
       // used.  
-#if defined(_WIN32) && !defined(__CYGWIN__)
-      char gsArgs[256];
-      sprintf(gsArgs,"-q -g%dx%d -r%dx%dx -sDEVICE=ppmraw "
-                     "-sOutputFile=%s.pnm -dNOPAUSE -dBATCH -- %s.ps",
-                     gx,gy,(int)(scaleFactor*72),(int)(scaleFactor*72),
-                     formBase.data(),formBase.data()
-             );
-      // gswin32 is a GUI api which will pop up a window and run
-      // asynchronously. To prevent both, we use ShellExecuteEx and
-      // WaitForSingleObject (thanks to Robert Golias for the code)
-      SHELLEXECUTEINFO sInfo = {
-        sizeof(SHELLEXECUTEINFO),   /* structure size */
-        SEE_MASK_NOCLOSEPROCESS,    /* leave the process running */
-        NULL,                       /* window handle */
-        NULL,                       /* action to perform: open */
-        "gswin32.exe",              /* file to execute */
-        gsArgs,                     /* argument list */ 
-        NULL,                       /* use current working dir */
-        SW_HIDE,                    /* minimize on start-up */
-        0,                          /* application instance handle */
-        NULL,                       /* ignored: id list */
-        NULL,                       /* ignored: class name */
-        NULL,                       /* ignored: key class */
-        0,                          /* ignored: hot key */
-        NULL,                       /* ignored: icon */
-        NULL                        /* resulting application handle */
-      };
-      if (!ShellExecuteEx(&sInfo))
-      {
-        err("Problem running ghostscript. Check your installation!\n");
-        return;
-      }
-      else if (sInfo.hProcess)      /* executable was launched, wait for it to finish */
-      {
-        WaitForSingleObject(sInfo.hProcess,INFINITE); 
-      }
-#else
+
       char gsArgs[4096];
       sprintf(gsArgs,"-q -g%dx%d -r%dx%dx -sDEVICE=ppmraw "
                     "-sOutputFile=%s.pnm -dNOPAUSE -dBATCH -- %s.ps",
                     gx,gy,(int)(scaleFactor*72),(int)(scaleFactor*72),
                     formBase.data(),formBase.data()
              );
-      if (iSystem("gs",gsArgs)!=0)
+      portable_sysTimerStart();
+      if (portable_system(portable_ghostScriptCommand(),gsArgs)!=0)
       {
-        err("Problem running ghostscript. Check your installation!\n");
+        err("Problem running ghostscript %s %s. Check your installation!\n",portable_ghostScriptCommand(),gsArgs);
+        portable_sysTimerStop();
+        QDir::setCurrent(oldDir);
         return;
       }
-#endif
+      portable_sysTimerStop();
       f.setName(formBase+".pnm");
       uint imageX=0,imageY=0;
       // we read the generated image again, to obtain the pixel data.
@@ -228,14 +201,14 @@ void FormulaList::generateBitmaps(const char *path)
         QTextStream t(&f);
         QCString s;
         if (!t.eof())
-          s=t.readLine();
+          s=t.readLine().utf8();
         if (s.length()<2 || s.left(2)!="P6")
-          err("Error: ghostscript produced an illegal image format!");
+          err("ghostscript produced an illegal image format!");
         else
         {
-          // assume the size if after the first line that does not start with
+          // assume the size is after the first line that does not start with
           // # excluding the first line of the file.
-          while (!t.eof() && (s=t.readLine()) && !s.isEmpty() && s.at(0)=='#');
+          while (!t.eof() && (s=t.readLine().utf8()) && !s.isEmpty() && s.at(0)=='#') { }
           sscanf(s,"%d %d",&imageX,&imageY);
         }
         if (imageX>0 && imageY>0)
@@ -297,7 +270,7 @@ void FormulaList::generateBitmaps(const char *path)
               dstImage.setPixel(x,y,QMIN(15,(c*15)/(16*10)));
             }
           }
-          // save the result as a png
+          // save the result as a bitmap
           QCString resultName;
           resultName.sprintf("form_%d.png",pageNum);
           // the option parameter 1 is used here as a temporary hack
@@ -314,19 +287,19 @@ void FormulaList::generateBitmaps(const char *path)
     }
     // remove intermediate files produced by latex
     thisDir.remove("_formulas.dvi");
-    thisDir.remove("_formulas.log");
+    if (!formulaError) thisDir.remove("_formulas.log"); // keep file in case of errors
     thisDir.remove("_formulas.aux");
   }
   // remove the latex file itself
   if (!formulaError) thisDir.remove("_formulas.tex");
   // write/update the formula repository so we know what text the 
-  // generated pngs represent (we use this next time to avoid regeneration
-  // of the pngs, and to avoid forcing the user to delete all pngs in order
+  // generated images represent (we use this next time to avoid regeneration
+  // of the images, and to avoid forcing the user to delete all images in order
   // to let a browser refresh the images).
   f.setName("formula.repository");
   if (f.open(IO_WriteOnly))
   {
-    QTextStream t(&f);
+    FTextStream t(&f);
     for (fli.toFirst();(formula=fli.current());++fli)
     {
       t << "\\form#" << formula->getId() << ":" << formula->getFormulaText() << endl;
