@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: index.h,v 1.28 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,9 +18,8 @@
 #ifndef INDEX_H
 #define INDEX_H
 
-#include "qtbc.h"
-#include <qfile.h>
 #include <qlist.h>
+#include <qcstring.h>
 
 class Definition;
 class MemberDef;
@@ -36,9 +35,11 @@ class IndexIntf
     virtual void finalize() = 0;
     virtual void incContentsDepth() = 0;
     virtual void decContentsDepth() = 0;
-    virtual void addContentsItem(bool isDir, const char *name, const char *ref = 0, 
-                                 const char *file = 0, const char *anchor = 0) = 0;
-    virtual void addIndexItem(Definition *context,MemberDef *md,const char *title) = 0;
+    virtual void addContentsItem(bool isDir, const char *name, const char *ref, 
+                                 const char *file, const char *anchor, bool separateIndex,
+                                 bool addToNavIndex,Definition *def) = 0;
+    virtual void addIndexItem(Definition *context,MemberDef *md,
+                              const char *sectionAnchor,const char *title) = 0;
     virtual void addIndexFile(const char *name) = 0;
     virtual void addImageFile(const char *name) = 0;
     virtual void addStyleSheetFile(const char *name) = 0;
@@ -90,12 +91,39 @@ class IndexList : public IndexIntf
       for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4,a5);
     }
 
+    template<typename A1,typename A2,typename A3,typename A4,typename A5,typename A6>
+    void foreach(void (IndexIntf::*methodPtr)(A1,A2,A3,A4,A5,A6),A1 a1,A2 a2,A3 a3,A4 a4,A5 a5,A6 a6)
+    {
+      QListIterator<IndexIntf> li(m_intfs);
+      for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4,a5,a6);
+    }
+
+    template<typename A1,typename A2,typename A3,typename A4,typename A5,typename A6,typename A7,typename A8>
+    void foreach(void (IndexIntf::*methodPtr)(A1,A2,A3,A4,A5,A6,A7,A8),A1 a1,A2 a2,A3 a3,A4 a4,A5 a5,A6 a6,A7 a7,A8 a8)
+    {
+      QListIterator<IndexIntf> li(m_intfs);
+      for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4,a5,a6,a7,a8);
+    }
+
+    template<typename A1,typename A2,typename A3,typename A4,typename A5,typename A6,typename A7,typename A8,typename A9>
+    void foreach(void (IndexIntf::*methodPtr)(A1,A2,A3,A4,A5,A6,A7,A8,A9),A1 a1,A2 a2,A3 a3,A4 a4,A5 a5,A6 a6,A7 a7,A8 a8,A9 a9)
+    {
+      QListIterator<IndexIntf> li(m_intfs);
+      for (li.toFirst();li.current();++li) (li.current()->*methodPtr)(a1,a2,a3,a4,a5,a6,a7,a8,a9);
+    }
+
   public:
     /** Creates a list of indexes */
-    IndexList() { m_intfs.setAutoDelete(TRUE); }
+    IndexList() { m_intfs.setAutoDelete(TRUE); m_enabled=TRUE; }
     /** Add an index generator to the list */
     void addIndex(IndexIntf *intf) 
     { m_intfs.append(intf); }
+    void disable()
+    { m_enabled = FALSE; }
+    void enable()
+    { m_enabled = TRUE; }
+    bool isEnabled() const
+    { return m_enabled; }
 
     // IndexIntf implementation
     void initialize() 
@@ -103,23 +131,26 @@ class IndexList : public IndexIntf
     void finalize() 
     { foreach(&IndexIntf::finalize); }
     void incContentsDepth()
-    { foreach(&IndexIntf::incContentsDepth); }
+    { if (m_enabled) foreach(&IndexIntf::incContentsDepth); }
     void decContentsDepth()
-    { foreach(&IndexIntf::decContentsDepth); }
-    void addContentsItem(bool isDir, const char *name, const char *ref = 0, 
-                         const char *file = 0, const char *anchor = 0)
-    { foreach<bool,const char *,const char *,const char *,const char*>
-             (&IndexIntf::addContentsItem,isDir,name,ref,file,anchor); }
-    void addIndexItem(Definition *context,MemberDef *md,const char *title=0)
-    { foreach<Definition *,MemberDef *>
-             (&IndexIntf::addIndexItem,context,md,title); }
+    { if (m_enabled) foreach(&IndexIntf::decContentsDepth); }
+    void addContentsItem(bool isDir, const char *name, const char *ref, 
+                         const char *file, const char *anchor,bool separateIndex=FALSE,bool addToNavIndex=FALSE,
+                         Definition *def=0)
+    { if (m_enabled) foreach<bool,const char *,const char *,const char *,const char*,bool,bool,Definition *>
+             (&IndexIntf::addContentsItem,isDir,name,ref,file,anchor,separateIndex,addToNavIndex,def); }
+    void addIndexItem(Definition *context,MemberDef *md,const char *sectionAnchor=0,const char *title=0)
+    { if (m_enabled) foreach<Definition *,MemberDef *,const char *,const char *>
+             (&IndexIntf::addIndexItem,context,md,sectionAnchor,title); }
     void addIndexFile(const char *name) 
-    { foreach<const char *>(&IndexIntf::addIndexFile,name); }
+    { if (m_enabled) foreach<const char *>(&IndexIntf::addIndexFile,name); }
     void addImageFile(const char *name) 
-    { foreach<const char *>(&IndexIntf::addImageFile,name); }
+    { if (m_enabled) foreach<const char *>(&IndexIntf::addImageFile,name); }
     void addStyleSheetFile(const char *name) 
-    { foreach<const char *>(&IndexIntf::addStyleSheetFile,name); }
+    { if (m_enabled) foreach<const char *>(&IndexIntf::addStyleSheetFile,name); }
 
+  private:
+    bool m_enabled;
 };
 
 
@@ -151,11 +182,19 @@ enum HighlightedItem
   HLI_None=0,
   HLI_Main,
   HLI_Modules,
-  HLI_Directories,
+  //HLI_Directories,
   HLI_Namespaces,
-  HLI_Hierarchy,
+  HLI_ClassHierarchy,
+  HLI_InterfaceHierarchy,
+  HLI_ExceptionHierarchy,
   HLI_Classes,
-  HLI_Annotated,
+  HLI_Interfaces,
+  HLI_Structs,
+  HLI_Exceptions,
+  HLI_AnnotatedClasses,
+  HLI_AnnotatedInterfaces,
+  HLI_AnnotatedStructs,
+  HLI_AnnotatedExceptions,
   HLI_Files,
   HLI_NamespaceMembers,
   HLI_Functions,
@@ -163,8 +202,12 @@ enum HighlightedItem
   HLI_Pages,
   HLI_Examples,
   HLI_Search,
+  HLI_UserGroup,
 
   HLI_ClassVisible,
+  HLI_InterfaceVisible,
+  HLI_StructVisible,
+  HLI_ExceptionVisible,
   HLI_NamespaceVisible,
   HLI_FileVisible
 };
@@ -189,6 +232,8 @@ enum FileMemberHighlight
   FMHL_Functions,
   FMHL_Variables,
   FMHL_Typedefs,
+  FMHL_Sequences,
+  FMHL_Dictionaries,
   FMHL_Enums,
   FMHL_EnumValues,
   FMHL_Defines,
@@ -201,9 +246,11 @@ enum NamespaceMemberHighlight
   NMHL_Functions,
   NMHL_Variables,
   NMHL_Typedefs,
+  NMHL_Sequences,
+  NMHL_Dictionaries,
   NMHL_Enums,
   NMHL_EnumValues,
-  NMHL_Total = FMHL_EnumValues+1
+  NMHL_Total = NMHL_EnumValues+1
 };
 
 enum ClassHighlight
@@ -219,33 +266,18 @@ enum ClassHighlight
   CHL_Total = CHL_Exceptions+1
 };
 
-void writeIndex(OutputList &ol);
-void writeHierarchicalIndex(OutputList &ol);
-void writeAlphabeticalIndex(OutputList &ol);
-void writeClassHierarchy(OutputList &ol);
-void writeAnnotatedIndex(OutputList &ol);
-void writeAnnotatedClassList(OutputList &ol);
-void writeMemberList(OutputList &ol,bool useSections);
-void writeSourceIndex(OutputList &ol);
-void writeHeaderIndex(OutputList &ol);
-void writeHeaderFileList(OutputList &ol);
-void writeExampleIndex(OutputList &ol);
-void writePageIndex(OutputList &ol);
-void writeFileIndex(OutputList &ol);
-void writeNamespaceIndex(OutputList &ol);
-void writeGroupIndex(OutputList &ol);
-void writeDirIndex(OutputList &ol);
-void writePackageIndex(OutputList &ol);
-void writeClassMemberIndex(OutputList &ol);
-void writeFileMemberIndex(OutputList &ol);
-void writeNamespaceMemberIndex(OutputList &ol);
-void writeGraphicalClassHierarchy(OutputList &ol);
 void writeGraphInfo(OutputList &ol);
+void writeIndexHierarchy(OutputList &ol);
 
 void countDataStructures();
 
 extern int annotatedClasses;
+extern int annotatedInterfaces;
+extern int annotatedStructs;
+extern int annotatedExceptions;
 extern int hierarchyClasses;
+extern int hierarchyInterfaces;
+extern int hierarchyExceptions;
 extern int documentedFiles;
 extern int documentedGroups;
 extern int documentedNamespaces;
@@ -253,16 +285,18 @@ extern int indexedPages;
 extern int documentedClassMembers[CMHL_Total];
 extern int documentedFileMembers[FMHL_Total];
 extern int documentedNamespaceMembers[NMHL_Total];
+extern int documentedDirs;
 extern int documentedHtmlFiles;
 extern int documentedPages;
-extern int documentedDirs;
 
 void startTitle(OutputList &ol,const char *fileName,Definition *def=0);
 void endTitle(OutputList &ol,const char *fileName,const char *name);
 void startFile(OutputList &ol,const char *name,const char *manName,
                const char *title,HighlightedItem hli=HLI_None,
-               bool additionalIndices=FALSE);
-void endFile(OutputList &ol,bool external=FALSE);
+               bool additionalIndices=FALSE,const char *altSidebarName=0);
+void endFile(OutputList &ol,bool skipNavIndex=FALSE,bool skipEndContents=FALSE,
+             const QCString &navPath=QCString());
+void endFileWithNavPath(Definition *d,OutputList &ol);
 
 void initClassMemberIndices();
 void initFileMemberIndices();
@@ -270,9 +304,6 @@ void initNamespaceMemberIndices();
 void addClassMemberNameToIndex(MemberDef *md);
 void addFileMemberNameToIndex(MemberDef *md);
 void addNamespaceMemberNameToIndex(MemberDef *md);
-
-// search engine
-void writeJavascriptSearchIndex();
-void writeSearchCategories(FTextStream &t);
+QCString fixSpaces(const QCString &s);
 
 #endif
