@@ -50,6 +50,9 @@ static const char layout_default[] =
 #define COMPILE_FOR_4_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3,langId4,text4) \
   COMPILE_FOR_3_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3)+ADD_OPTION(langId4,text4)
 
+#define COMPILE_FOR_5_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3,langId4,text4,langId5,text5) \
+  COMPILE_FOR_4_OPTIONS(def,langId1,text1,langId2,text2,langId3,text3,langId4,text4)+ADD_OPTION(langId5,text5)
+
 static bool elemIsVisible(const QXmlAttributes &attrib,bool defVal=TRUE)
 {
   QCString visible = attrib.value("visible").utf8();
@@ -57,10 +60,10 @@ static bool elemIsVisible(const QXmlAttributes &attrib,bool defVal=TRUE)
   if (visible.at(0)=='$' && visible.length()>1)
   {
     QCString id = visible.mid(1);
-    ConfigOption *opt = Config::instance()->get(id);
-    if (opt && opt->kind()==ConfigOption::O_Bool)
+    const ConfigValues::Info *opt = ConfigValues::instance().get(id);
+    if (opt && opt->type==ConfigValues::Info::Bool)
     {
-      return *((ConfigBool *)opt)->valueRef();
+      return ConfigValues::instance().*((ConfigValues::InfoBool*)opt)->item;
     }
     else if (!opt)
     {
@@ -120,7 +123,7 @@ QCString LayoutNavEntry::url() const
     }
     if (!found)
     {
-      msg("explicit link request to '%s' in layout file '%s' could not be resolved\n",qPrint(url.mid(5)),qPrint(Config_getString("LAYOUT_FILE")));
+      msg("explicit link request to '%s' in layout file '%s' could not be resolved\n",qPrint(url.mid(5)),qPrint(Config_getString(LAYOUT_FILE)));
     }
   }
   //printf("LayoutNavEntry::url()=%s\n",url.data());
@@ -262,9 +265,10 @@ class LayoutParser : public QXmlDefaultHandler
       m_part = -1; // invalid
       m_rootNav = 0;
 
-      //bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
-      //bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
-      //bool javaOpt    = Config_getBool("OPTIMIZE_OUTPUT_JAVA");
+      //bool fortranOpt = Config_getBool(OPTIMIZE_FOR_FORTRAN);
+      //bool vhdlOpt    = Config_getBool(OPTIMIZE_OUTPUT_VHDL);  
+      //bool javaOpt    = Config_getBool(OPTIMIZE_OUTPUT_JAVA);
+      bool sliceOpt    = Config_getBool(OPTIMIZE_OUTPUT_SLICE);
 
       // start & end handlers
       m_sHandler.insert("doxygenlayout", 
@@ -310,7 +314,7 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandlerSection(this,LayoutDocEntry::ClassNestedClasses,&LayoutParser::startSectionEntry,
                                          COMPILE_FOR_2_OPTIONS(
                                            theTranslator->trCompounds(),
-                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_VHDL,theTranslator->trVhdlType(VhdlDocGen::ENTITY,FALSE),
                                            SrcLangExt_Fortran,theTranslator->trDataTypes()
                                          )));
       m_sHandler.insert("class/memberdecl/services",
@@ -331,9 +335,10 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("class/memberdecl/publicmethods", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_pubMethods,
-                                        COMPILE_FOR_1_OPTION(
+                                        COMPILE_FOR_2_OPTIONS(
                                           theTranslator->trPublicMembers(),
-                                          SrcLangExt_ObjC,theTranslator->trInstanceMethods()
+                                          SrcLangExt_ObjC,theTranslator->trInstanceMethods(),
+                                          SrcLangExt_Slice,theTranslator->trOperations()
                                         ))); 
       m_sHandler.insert("class/memberdecl/publicstaticmethods", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
@@ -344,7 +349,11 @@ class LayoutParser : public QXmlDefaultHandler
                                         ))); 
       m_sHandler.insert("class/memberdecl/publicattributes", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
-                                        MemberListType_pubAttribs,theTranslator->trPublicAttribs())); 
+                                        MemberListType_pubAttribs,
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trPublicAttribs(),
+                                          SrcLangExt_Slice,theTranslator->trDataMembers()
+                                        ))); 
       m_sHandler.insert("class/memberdecl/publicstaticattributes", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_pubStaticAttribs,theTranslator->trStaticPublicAttribs())); 
@@ -440,17 +449,22 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("class/memberdef/functions", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_functionMembers,
-                                        COMPILE_FOR_2_OPTIONS(
+                                        COMPILE_FOR_3_OPTIONS(
                                           theTranslator->trMemberFunctionDocumentation(),
                                           SrcLangExt_ObjC,theTranslator->trMethodDocumentation(),
-                                          SrcLangExt_Fortran,theTranslator->trMemberFunctionDocumentationFortran()
+                                          SrcLangExt_Fortran,theTranslator->trMemberFunctionDocumentationFortran(),
+                                          SrcLangExt_Slice,theTranslator->trOperationDocumentation()
                                         )));
       m_sHandler.insert("class/memberdef/related", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_relatedMembers,theTranslator->trRelatedFunctionDocumentation()));
       m_sHandler.insert("class/memberdef/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
-                                        MemberListType_variableMembers,theTranslator->trMemberDataDocumentation()));
+                                        MemberListType_variableMembers,
+                                        COMPILE_FOR_1_OPTION(
+                                          theTranslator->trMemberDataDocumentation(),
+                                          SrcLangExt_Slice,theTranslator->trDataMemberDocumentation()
+                                        ))); 
       m_sHandler.insert("class/memberdef/properties", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_propertyMembers,theTranslator->trPropertyDocumentation()));
@@ -477,28 +491,45 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandler(this,&LayoutParser::startMemberDecl));
       m_sHandler.insert("namespace/memberdecl/nestednamespaces", 
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceNestedNamespaces,&LayoutParser::startSectionEntry,
-                                         COMPILE_FOR_4_OPTIONS(
+                                         COMPILE_FOR_5_OPTIONS(
                                            theTranslator->trNamespaces(),
                                            SrcLangExt_Java,theTranslator->trPackages(),
                                            SrcLangExt_VHDL,theTranslator->trPackages(),
                                            SrcLangExt_IDL,theTranslator->trModules(),
-                                           SrcLangExt_Fortran,theTranslator->trModules()
-                                         )));
+                                           SrcLangExt_Fortran,theTranslator->trModules(),
+                                           SrcLangExt_Slice,(sliceOpt ?
+                                                             theTranslator->trModules() :
+                                                             theTranslator->trNamespaces()))));
       m_sHandler.insert("namespace/memberdecl/constantgroups",
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceNestedConstantGroups,&LayoutParser::startSectionEntry,
                                          theTranslator->trConstantGroups()));
+      m_sHandler.insert("namespace/memberdecl/interfaces", 
+          new StartElementHandlerSection(this,LayoutDocEntry::NamespaceInterfaces,&LayoutParser::startSectionEntry,
+                                         theTranslator->trSliceInterfaces()));
       m_sHandler.insert("namespace/memberdecl/classes", 
           new StartElementHandlerSection(this,LayoutDocEntry::NamespaceClasses,&LayoutParser::startSectionEntry,
                                          COMPILE_FOR_2_OPTIONS(
                                            theTranslator->trCompounds(),
-                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_VHDL,theTranslator->trVhdlType(VhdlDocGen::ENTITY,FALSE),
                                            SrcLangExt_Fortran,theTranslator->trDataTypes()
                                          )));
+      m_sHandler.insert("namespace/memberdecl/structs", 
+          new StartElementHandlerSection(this,LayoutDocEntry::NamespaceStructs,&LayoutParser::startSectionEntry,
+                                         theTranslator->trStructs()));
+      m_sHandler.insert("namespace/memberdecl/exceptions", 
+          new StartElementHandlerSection(this,LayoutDocEntry::NamespaceExceptions,&LayoutParser::startSectionEntry,
+                                         theTranslator->trExceptions()));
       m_sHandler.insert("namespace/memberdecl/membergroups", 
           new StartElementHandlerKind(this,LayoutDocEntry::MemberGroups,&LayoutParser::startSimpleEntry));
       m_sHandler.insert("namespace/memberdecl/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decTypedefMembers,theTranslator->trTypedefs()));
+      m_sHandler.insert("namespace/memberdecl/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decSequenceMembers,theTranslator->trSequences()));
+      m_sHandler.insert("namespace/memberdecl/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decDictionaryMembers,theTranslator->trDictionaries()));
       m_sHandler.insert("namespace/memberdecl/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decEnumMembers,theTranslator->trEnumerations()));
@@ -508,11 +539,12 @@ class LayoutParser : public QXmlDefaultHandler
                                         COMPILE_FOR_2_OPTIONS(
                                           theTranslator->trFunctions(),
                                           SrcLangExt_Fortran,theTranslator->trSubprograms(),
-                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                          SrcLangExt_VHDL,theTranslator->trFunctionAndProc()
                                         )));
       m_sHandler.insert("namespace/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
-                                        MemberListType_decVarMembers,theTranslator->trVariables()));
+                                        MemberListType_decVarMembers,
+                                        sliceOpt ? theTranslator->trConstants() : theTranslator->trVariables()));
       m_eHandler.insert("namespace/memberdecl", 
           new EndElementHandler(this,&LayoutParser::endMemberDecl));
       m_sHandler.insert("namespace/memberdef", 
@@ -526,6 +558,13 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("namespace/memberdef/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docTypedefMembers,theTranslator->trTypedefDocumentation()));
+      m_sHandler.insert("namespace/memberdef/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docSequenceMembers,theTranslator->trSequenceDocumentation()));
+      m_sHandler.insert("namespace/memberdef/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docDictionaryMembers,
+                                        theTranslator->trDictionaryDocumentation()));
       m_sHandler.insert("namespace/memberdef/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docEnumMembers,theTranslator->trEnumerationTypeDocumentation()));
@@ -538,7 +577,9 @@ class LayoutParser : public QXmlDefaultHandler
                                         )));
       m_sHandler.insert("namespace/memberdef/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
-                                        MemberListType_docVarMembers,theTranslator->trVariableDocumentation()));
+                                        MemberListType_docVarMembers,
+                                        sliceOpt ? theTranslator->trConstantDocumentation() :
+                                        theTranslator->trVariableDocumentation()));
       m_eHandler.insert("namespace/memberdef", 
           new EndElementHandler(this,&LayoutParser::endMemberDef));
       m_eHandler.insert("namespace", 
@@ -566,20 +607,30 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandlerKind(this,LayoutDocEntry::MemberGroups,&LayoutParser::startSimpleEntry));
       m_sHandler.insert("file/memberdecl", 
           new StartElementHandler(this,&LayoutParser::startMemberDecl));
+      m_sHandler.insert("file/memberdecl/interfaces", 
+          new StartElementHandlerSection(this,LayoutDocEntry::FileInterfaces,&LayoutParser::startSectionEntry,
+                                         theTranslator->trSliceInterfaces()));
       m_sHandler.insert("file/memberdecl/classes", 
           new StartElementHandlerSection(this,LayoutDocEntry::FileClasses,&LayoutParser::startSectionEntry,
                                          COMPILE_FOR_2_OPTIONS(
                                            theTranslator->trCompounds(),
-                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_VHDL,theTranslator->trVhdlType(VhdlDocGen::ENTITY,FALSE),
                                            SrcLangExt_Fortran,theTranslator->trDataTypes()
                                          )));
+      m_sHandler.insert("file/memberdecl/structs", 
+          new StartElementHandlerSection(this,LayoutDocEntry::FileStructs,&LayoutParser::startSectionEntry,
+                                         theTranslator->trStructs()));
+      m_sHandler.insert("file/memberdecl/exceptions", 
+          new StartElementHandlerSection(this,LayoutDocEntry::FileExceptions,&LayoutParser::startSectionEntry,
+                                         theTranslator->trExceptions()));
       m_sHandler.insert("file/memberdecl/namespaces", 
           new StartElementHandlerSection(this,LayoutDocEntry::FileNamespaces,&LayoutParser::startSectionEntry,
-                                         COMPILE_FOR_3_OPTIONS(
+                                         COMPILE_FOR_4_OPTIONS(
                                            theTranslator->trNamespaces(),
                                            SrcLangExt_Java,theTranslator->trPackages(),
                                            SrcLangExt_IDL,theTranslator->trModules(),
-                                           SrcLangExt_Fortran,theTranslator->trModules()
+                                           SrcLangExt_Fortran,theTranslator->trModules(),
+                                           SrcLangExt_Slice,theTranslator->trModules()
                                          )));
       m_sHandler.insert("file/memberdecl/constantgroups",
           new StartElementHandlerSection(this,LayoutDocEntry::FileConstantGroups,&LayoutParser::startSectionEntry,
@@ -590,6 +641,12 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("file/memberdecl/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decTypedefMembers,theTranslator->trTypedefs()));
+      m_sHandler.insert("file/memberdecl/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decSequenceMembers,theTranslator->trSequences()));
+      m_sHandler.insert("file/memberdecl/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decDictionaryMembers,theTranslator->trDictionaries()));
       m_sHandler.insert("file/memberdecl/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decEnumMembers,theTranslator->trEnumerations()));
@@ -599,11 +656,12 @@ class LayoutParser : public QXmlDefaultHandler
                                         COMPILE_FOR_2_OPTIONS(
                                           theTranslator->trFunctions(),
                                           SrcLangExt_Fortran,theTranslator->trSubprograms(),
-                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                          SrcLangExt_VHDL,theTranslator->trFunctionAndProc()
                                         )));
       m_sHandler.insert("file/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
-                                        MemberListType_decVarMembers,theTranslator->trVariables()));
+                                        MemberListType_decVarMembers,
+                                        sliceOpt ? theTranslator->trConstants() : theTranslator->trVariables()));
 
       m_eHandler.insert("file/memberdecl", 
           new EndElementHandler(this,&LayoutParser::endMemberDecl));
@@ -621,6 +679,13 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("file/memberdef/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docTypedefMembers,theTranslator->trTypedefDocumentation()));
+      m_sHandler.insert("file/memberdef/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docSequenceMembers,theTranslator->trSequenceDocumentation()));
+      m_sHandler.insert("file/memberdef/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docDictionaryMembers,
+                                        theTranslator->trDictionaryDocumentation()));
       m_sHandler.insert("file/memberdef/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docEnumMembers,
@@ -660,7 +725,7 @@ class LayoutParser : public QXmlDefaultHandler
           new StartElementHandlerSection(this,LayoutDocEntry::GroupClasses,&LayoutParser::startSectionEntry,
                                          COMPILE_FOR_2_OPTIONS(
                                            theTranslator->trCompounds(),
-                                           SrcLangExt_VHDL,VhdlDocGen::trVhdlType(VhdlDocGen::ENTITY,FALSE),
+                                           SrcLangExt_VHDL,theTranslator->trVhdlType(VhdlDocGen::ENTITY,FALSE),
                                            SrcLangExt_Fortran,theTranslator->trDataTypes()
                                          )));
       m_sHandler.insert("group/memberdecl/namespaces", 
@@ -689,6 +754,12 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("group/memberdecl/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decTypedefMembers,theTranslator->trTypedefs()));
+      m_sHandler.insert("group/memberdecl/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decSequenceMembers,theTranslator->trSequences()));
+      m_sHandler.insert("group/memberdecl/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
+                                        MemberListType_decDictionaryMembers,theTranslator->trDictionaries()));
       m_sHandler.insert("group/memberdecl/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
                                         MemberListType_decEnumMembers,theTranslator->trEnumerations()));
@@ -701,7 +772,7 @@ class LayoutParser : public QXmlDefaultHandler
                                         COMPILE_FOR_2_OPTIONS(
                                           theTranslator->trFunctions(),
                                           SrcLangExt_Fortran,theTranslator->trSubprograms(),
-                                          SrcLangExt_VHDL,VhdlDocGen::trFunctionAndProc()
+                                          SrcLangExt_VHDL,theTranslator->trFunctionAndProc()
                                         )));
       m_sHandler.insert("group/memberdecl/variables", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDeclEntry,
@@ -745,6 +816,13 @@ class LayoutParser : public QXmlDefaultHandler
       m_sHandler.insert("group/memberdef/typedefs", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docTypedefMembers,theTranslator->trTypedefDocumentation()));
+      m_sHandler.insert("group/memberdef/sequences", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docSequenceMembers,theTranslator->trSequenceDocumentation()));
+      m_sHandler.insert("group/memberdef/dictionaries", 
+          new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
+                                        MemberListType_docDictionaryMembers,
+                                        theTranslator->trDictionaryDocumentation()));
       m_sHandler.insert("group/memberdef/enums", 
           new StartElementHandlerMember(this,&LayoutParser::startMemberDefEntry,
                                         MemberListType_docEnumMembers,theTranslator->trEnumerationTypeDocumentation()));
@@ -887,19 +965,20 @@ class LayoutParser : public QXmlDefaultHandler
       {
         // no MainPage node... add one as the first item of the root node...
         new LayoutNavEntry(m_rootNav,LayoutNavEntry::MainPage, TRUE, 
-            /*Config_getBool("GENERATE_TREEVIEW") ? "main" :*/ "index",
-            theTranslator->trMainPage(),TRUE);
+            /*Config_getBool(GENERATE_TREEVIEW) ? "main" :*/ "index",
+            theTranslator->trMainPage(),"",TRUE);
       }
     }
 
     void startNavEntry(const QXmlAttributes &attrib)
     {
-      static bool javaOpt    = Config_getBool("OPTIMIZE_OUTPUT_JAVA");
-      static bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
-      static bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
-      static bool hasGraphicalHierarchy = Config_getBool("HAVE_DOT") &&
-                                          Config_getBool("GRAPHICAL_HIERARCHY");
-      static bool extractAll = Config_getBool("EXTRACT_ALL");
+      static bool javaOpt    = Config_getBool(OPTIMIZE_OUTPUT_JAVA);
+      static bool fortranOpt = Config_getBool(OPTIMIZE_FOR_FORTRAN);
+      static bool vhdlOpt    = Config_getBool(OPTIMIZE_OUTPUT_VHDL);  
+      static bool sliceOpt   = Config_getBool(OPTIMIZE_OUTPUT_SLICE);  
+      static bool hasGraphicalHierarchy = Config_getBool(HAVE_DOT) &&
+                                          Config_getBool(GRAPHICAL_HIERARCHY);
+      static bool extractAll = Config_getBool(EXTRACT_ALL);
       static struct NavEntryMap
       {
         const char *typeStr;       // type attribute name in the XML file
@@ -933,60 +1012,137 @@ class LayoutParser : public QXmlDefaultHandler
         },
         { "namespaces",
           LayoutNavEntry::Namespaces,
-          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModules() : theTranslator->trNamespaces(),
-          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
-          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt || sliceOpt ? theTranslator->trModules() : theTranslator->trNamespaces(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt || sliceOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt || sliceOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
           "namespaces"
         },
         { "namespacelist",
           LayoutNavEntry::NamespaceList,
-          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackages() : fortranOpt || sliceOpt ? theTranslator->trModulesList() : theTranslator->trNamespaceList(),
           QCString(),
-          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageListDescription() : fortranOpt || sliceOpt ? theTranslator->trModulesListDescription(extractAll) : theTranslator->trNamespaceListDescription(extractAll),
           "namespaces"
         },
         { "namespacemembers",
           LayoutNavEntry::NamespaceMembers,
-          javaOpt || vhdlOpt   ? theTranslator->trPackageMembers() : fortranOpt ? theTranslator->trModulesMembers() : theTranslator->trNamespaceMembers(),
+          javaOpt || vhdlOpt   ? theTranslator->trPackageMembers() : fortranOpt || sliceOpt ? theTranslator->trModulesMembers() : theTranslator->trNamespaceMembers(),
           QCString(),
-          fortranOpt ? theTranslator->trModulesMemberDescription(extractAll) : theTranslator->trNamespaceMemberDescription(extractAll),
+          fortranOpt || sliceOpt ? theTranslator->trModulesMemberDescription(extractAll) : theTranslator->trNamespaceMemberDescription(extractAll),
           "namespacemembers"
         },
         { "classindex",
           LayoutNavEntry::ClassIndex,
-          fortranOpt ? theTranslator->trDataTypes() : vhdlOpt ? VhdlDocGen::trDesignUnits() : theTranslator->trCompoundIndex(),
+          fortranOpt ? theTranslator->trDataTypes() : vhdlOpt ? theTranslator->trDesignUnits() : theTranslator->trCompoundIndex(),
           QCString(),
           QCString(),
           "classes"
         },
         { "classes",
           LayoutNavEntry::Classes,
-          fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitList() : theTranslator->trClasses(),
+          fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? theTranslator->trDesignUnitList() : theTranslator->trClasses(),
           theTranslator->trCompoundList(),
-          fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
+          fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? theTranslator->trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
           "annotated"
         },
         { "classlist",
           LayoutNavEntry::ClassList,
-          fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitList() : theTranslator->trCompoundList(),
+          fortranOpt ? theTranslator->trCompoundListFortran() : vhdlOpt ? theTranslator->trDesignUnitList() : theTranslator->trCompoundList(),
           QCString(),
-          fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
+          fortranOpt ? theTranslator->trCompoundListDescriptionFortran() : vhdlOpt ? theTranslator->trDesignUnitListDescription() : theTranslator->trCompoundListDescription(),
           "annotated"
         },
         { "hierarchy",
           LayoutNavEntry::ClassHierarchy,
-          vhdlOpt    ? VhdlDocGen::trDesignUnitHierarchy() : theTranslator->trClassHierarchy(),
+          vhdlOpt    ? theTranslator->trDesignUnitHierarchy() : theTranslator->trClassHierarchy(),
           QCString(),
           theTranslator->trClassHierarchyDescription(),
           hasGraphicalHierarchy ? "inherits" : "hierarchy"
         },
         { "classmembers",
           LayoutNavEntry::ClassMembers,
-          fortranOpt ? theTranslator->trCompoundMembersFortran() : vhdlOpt ? VhdlDocGen::trDesignUnitMembers() : theTranslator->trCompoundMembers(),
+          fortranOpt ? theTranslator->trCompoundMembersFortran() : vhdlOpt ? theTranslator->trDesignUnitMembers() : theTranslator->trCompoundMembers(),
           QCString(),
           fortranOpt ? theTranslator->trCompoundMembersDescriptionFortran(extractAll) : theTranslator->trCompoundMembersDescription(extractAll),
           "functions"
         },
+        { "interfaceindex",
+          LayoutNavEntry::InterfaceIndex,
+          theTranslator->trInterfaceIndex(),
+          QCString(),
+          QCString(),
+          "interfaces"
+        },
+        { "interfaces",
+          LayoutNavEntry::Interfaces,
+          theTranslator->trSliceInterfaces(),
+          theTranslator->trInterfaceList(),
+          theTranslator->trInterfaceListDescription(),
+          "annotatedinterfaces"
+        },
+        { "interfacelist",
+          LayoutNavEntry::InterfaceList,
+          theTranslator->trInterfaceList(),
+          QCString(),
+          theTranslator->trInterfaceListDescription(),
+          "annotatedinterfaces"
+        },
+        { "interfacehierarchy",
+          LayoutNavEntry::InterfaceHierarchy,
+          theTranslator->trInterfaceHierarchy(),
+          QCString(),
+          theTranslator->trInterfaceHierarchyDescription(),
+          hasGraphicalHierarchy ? "interfaceinherits" : "interfacehierarchy"
+        },
+        { "structindex",
+          LayoutNavEntry::StructIndex,
+          theTranslator->trStructIndex(),
+          QCString(),
+          QCString(),
+          "structs"
+        },
+        { "structs",
+          LayoutNavEntry::Structs,
+          theTranslator->trStructs(),
+          theTranslator->trStructList(),
+          theTranslator->trStructListDescription(),
+          "annotatedstructs"
+        },
+        { "structlist",
+          LayoutNavEntry::StructList,
+          theTranslator->trStructList(),
+          QCString(),
+          theTranslator->trStructListDescription(),
+          "annotatedstructs"
+        },
+        { "exceptionindex",
+          LayoutNavEntry::ExceptionIndex,
+          theTranslator->trExceptionIndex(),
+          QCString(),
+          QCString(),
+          "exceptions"
+        },
+        { "exceptions",
+          LayoutNavEntry::Exceptions,
+          theTranslator->trExceptions(),
+          theTranslator->trExceptionList(),
+          theTranslator->trExceptionListDescription(),
+          "annotatedexceptions"
+        },
+        { "exceptionlist",
+          LayoutNavEntry::ExceptionList,
+          theTranslator->trExceptionList(),
+          QCString(),
+          theTranslator->trExceptionListDescription(),
+          "annotatedexceptions"
+        },
+        { "exceptionhierarchy",
+          LayoutNavEntry::ExceptionHierarchy,
+          theTranslator->trExceptionHierarchy(),
+          QCString(),
+          theTranslator->trExceptionHierarchyDescription(),
+          hasGraphicalHierarchy ? "exceptioninherits" : "exceptionhierarchy"
+        },
         { "files",
           LayoutNavEntry::Files,
           theTranslator->trFile(TRUE,FALSE),
@@ -1268,7 +1424,8 @@ class LayoutParser : public QXmlDefaultHandler
     }
 
   private:
-    LayoutParser() : m_sHandler(163), m_eHandler(17), m_invalidEntry(FALSE) { }
+    LayoutParser() : m_sHandler(163), m_eHandler(17), m_invalidEntry(FALSE), m_part(0), m_rootNav(NULL) { }
+   ~LayoutParser() { delete m_rootNav; }
 
     QDict<StartElementHandler> m_sHandler;
     QDict<EndElementHandler>   m_eHandler;
@@ -1396,7 +1553,8 @@ void LayoutDocManager::parse(QTextStream &t,const char *fileName)
 void writeDefaultLayoutFile(const char *fileName)
 {
   QFile f(fileName);
-  if (!f.open(IO_WriteOnly))
+  bool ok = openOutputFile(fileName,f);
+  if (!ok)
   {
     err("Failed to open file %s for writing!\n",fileName);
     return;
