@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: layout.h,v 1.2 2001/03/19 19:27:41 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -19,11 +19,12 @@
 #ifndef LAYOUT_H
 #define LAYOUT_H
 
-#include "qtbc.h"
-#include "memberlist.h"
 #include <qlist.h>
+#include "types.h"
 
 class LayoutParser;
+class MemberList;
+class QTextStream;
 
 /** @brief Base class representing a piece of a documentation page */
 struct LayoutDocEntry
@@ -44,11 +45,12 @@ struct LayoutDocEntry
               ClassUsedFiles,
 
               // Namespace specific items
-              NamespaceNestedNamespaces, NamespaceClasses,
+              NamespaceNestedNamespaces, NamespaceNestedConstantGroups,
+              NamespaceClasses, NamespaceInterfaces, NamespaceStructs, NamespaceExceptions,
               NamespaceInlineClasses,
 
               // File specific items
-              FileClasses, FileNamespaces, 
+              FileClasses, FileInterfaces, FileStructs, FileExceptions, FileConstantGroups, FileNamespaces,
               FileIncludes, FileIncludeGraph, 
               FileIncludedByGraph, FileSourceLink,
               FileInlineClasses,
@@ -87,12 +89,12 @@ private:
 /** @brief Represents of a member declaration list with configurable title and subtitle. */
 struct LayoutDocEntryMemberDecl: public LayoutDocEntry
 {
-  LayoutDocEntryMemberDecl(MemberList::ListType tp,
+  LayoutDocEntryMemberDecl(MemberListType tp,
                            const QCString &tl,const QCString &ss) 
     : type(tp), m_title(tl), m_subscript(ss) {}
 
   Kind kind() const { return MemberDecl; }
-  MemberList::ListType type;
+  MemberListType type;
   QCString title(SrcLangExt lang) const;
   QCString subtitle(SrcLangExt lang) const;
 private:
@@ -103,11 +105,11 @@ private:
 /** @brief Represents of a member definition list with configurable title. */
 struct LayoutDocEntryMemberDef: public LayoutDocEntry
 {
-  LayoutDocEntryMemberDef(MemberList::ListType tp,const QCString &tl) 
+  LayoutDocEntryMemberDef(MemberListType tp,const QCString &tl) 
     : type(tp), m_title(tl) {}
 
   Kind kind() const { return MemberDef; }
-  MemberList::ListType type;
+  MemberListType type;
   QCString title(SrcLangExt lang) const;
 private:
   QCString m_title;
@@ -118,6 +120,7 @@ struct LayoutNavEntry
 {
   public:
     enum Kind { 
+      None = -1, 
       MainPage, 
       Pages,
       Modules, 
@@ -129,10 +132,20 @@ struct LayoutNavEntry
       ClassIndex, 
       ClassHierarchy, 
       ClassMembers,
+      Interfaces,
+      InterfaceList, 
+      InterfaceIndex, 
+      InterfaceHierarchy, 
+      Structs,
+      StructList, 
+      StructIndex, 
+      Exceptions,
+      ExceptionList, 
+      ExceptionIndex, 
+      ExceptionHierarchy, 
       Files, 
       FileList,
       FileGlobals,
-      //Dirs, 
       Examples,
       User,
       UserGroup
@@ -157,7 +170,7 @@ struct LayoutNavEntry
     LayoutNavEntry *find(LayoutNavEntry::Kind k,const char *file=0) const;
 
   private:
-    LayoutNavEntry() : m_parent(0) {}
+    LayoutNavEntry() : m_parent(0), m_kind(None), m_visible(FALSE) {}
     LayoutNavEntry *m_parent;
     Kind m_kind;
     bool m_visible;
