@@ -62,7 +62,7 @@ class ManGenerator : public OutputGenerator
     void endTitle();
     
     void newParagraph();
-    void startParagraph();
+    void startParagraph(const char *classDef);
     void endParagraph();
     void writeString(const char *text);
     void startIndexListItem() {}
@@ -99,9 +99,10 @@ class ManGenerator : public OutputGenerator
     void endMemberSections() {}
     void startHeaderSection() {}
     void endHeaderSection();
-    void startMemberHeader(const char *);
+    void startMemberHeader(const char *,int);
     void endMemberHeader();
     void insertMemberAlign(bool) {}
+    void insertMemberAlignLeft(int,bool){}
     void startMemberSubtitle() {}
     void endMemberSubtitle() {}
     //void writeListItem();
@@ -160,7 +161,7 @@ class ManGenerator : public OutputGenerator
     void endCenter()          {}
     void startSmall()         {}
     void endSmall()           {}
-    void startMemberDescription(const char *,const char *) { t << "\n.RI \""; firstCol=FALSE; }
+    void startMemberDescription(const char *,const char *,bool) { t << "\n.RI \""; firstCol=FALSE; }
     void endMemberDescription()   { t << "\""; firstCol=FALSE; }
     void startMemberDeclaration() {} 
     void endMemberDeclaration(const char *,const char *) {}
@@ -168,8 +169,8 @@ class ManGenerator : public OutputGenerator
                       const char *,const char *,const char *) {}
     void startDescList(SectionTypes);
     void endDescList()        {}
-    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
-    void endSimpleSect();
+    void startExamples();
+    void endExamples();
     void startParamList(ParamListTypes,const char *title);
     void endParamList();
     //void writeDescItem();
@@ -196,9 +197,8 @@ class ManGenerator : public OutputGenerator
     void endContents() {}
     void writeNonBreakableSpace(int n) { int i; for (i=0;i<n;i++) t << " "; }
     
-    void startDescTable(const char *t) 
-    { startSimpleSect(EnumValues,0,0,t); startDescForItem(); }
-    void endDescTable() { endDescForItem(); endSimpleSect(); }
+    void startDescTable(const char *t);
+    void endDescTable();
     void startDescTableRow() {}
     void endDescTableRow() {}
     void startDescTableTitle() { startItemListItem(); startBold(); startEmphasis(); endItemListItem(); }
