@@ -9,6 +9,8 @@
 #include "definition.h"
 #include "groupdef.h"
 #include "example.h"
+#include "arguments.h"
+#include "doxygen.h"
 
 #define HEADER ('D'<<24)+('O'<<16)+('X'<<8)+'!'
 
@@ -32,6 +34,12 @@ void marshalUInt(StorageIntf *s,uint v)
   s->write((const char *)b,4);
 }
 
+void marshalUInt64(StorageIntf *s,uint64 v)
+{
+  marshalUInt(s, uint(v>>32));
+  marshalUInt(s, uint(v&0xFFFFFFFF));
+}
+
 void marshalBool(StorageIntf *s,bool b)
 {
   char c = b;
@@ -54,32 +62,7 @@ void marshalQGString(StorageIntf *s,const QGString &str)
 
 void marshalArgumentList(StorageIntf *s,ArgumentList *argList)
 {
-  if (argList==0)
-  {
-    marshalUInt(s,NULL_LIST); // null pointer representation
-  }
-  else
-  {
-    marshalUInt(s,argList->count());
-    if (argList->count()>0)
-    {
-      ArgumentListIterator ali(*argList);
-      Argument *a;
-      for (ali.toFirst();(a=ali.current());++ali)
-      {
-        marshalQCString(s,a->attrib);    
-        marshalQCString(s,a->type);    
-        marshalQCString(s,a->canType);    
-        marshalQCString(s,a->name);    
-        marshalQCString(s,a->array);    
-        marshalQCString(s,a->defval);    
-        marshalQCString(s,a->docs);    
-      }
-    }
-    marshalBool(s,argList->constSpecifier);
-    marshalBool(s,argList->volatileSpecifier);
-    marshalBool(s,argList->pureSpecifier);
-  }
+  ArgumentList::marshal(s,argList);
 }
 
 void marshalArgumentLists(StorageIntf *s,QList<ArgumentList> *argLists)
@@ -157,7 +140,10 @@ void marshalSectionInfoList(StorageIntf *s, QList<SectionInfo> *anchors)
       marshalQCString(s,si->ref);
       marshalInt(s,(int)si->type);
       marshalQCString(s,si->fileName);
+      marshalInt(s,si->lineNr);
+      marshalInt(s,si->level);
     }
+    anchors->clear();
   }
 }
 
@@ -195,7 +181,7 @@ void marshalSectionDict(StorageIntf *s,SectionDict *sections)
   else
   {
     marshalUInt(s,sections->count());
-    QDictIterator<SectionInfo> sli(*sections);
+    SDict<SectionInfo>::IteratorDict sli(*sections);
     SectionInfo *si;
     for (sli.toFirst();(si=sli.current());++sli)
     {
@@ -360,6 +346,15 @@ void marshalMemberLists(StorageIntf *s,SDict<MemberList> *mls)
   }
 }
 
+void marshalLocalToc(StorageIntf *s,const LocalToc &lt)
+{
+  marshalInt(s,lt.mask());
+  marshalInt(s,lt.htmlLevel());
+  marshalInt(s,lt.latexLevel());
+  marshalInt(s,lt.xmlLevel());
+  marshalInt(s,lt.docbookLevel());
+}
+
 void marshalEntry(StorageIntf *s,Entry *e)
 {
   marshalUInt(s,HEADER);
@@ -368,14 +363,17 @@ void marshalEntry(StorageIntf *s,Entry *e)
   marshalInt(s,e->section);
   marshalInt(s,(int)e->protection);
   marshalInt(s,(int)e->mtype);
-  marshalInt(s,e->spec);
+  marshalUInt64(s,e->spec);
   marshalInt(s,e->initLines);
   marshalBool(s,e->stat);
+  marshalLocalToc(s,e->localToc);
   marshalBool(s,e->explicitExternal);
   marshalBool(s,e->proto);
   marshalBool(s,e->subGrouping);
   marshalBool(s,e->callGraph);
   marshalBool(s,e->callerGraph);
+  marshalBool(s,e->referencedByRelation);
+  marshalBool(s,e->referencesRelation);
   marshalInt(s,(int)e->virt);
   marshalQCString(s,e->args);
   marshalQCString(s,e->bitfields);
@@ -414,6 +412,8 @@ void marshalEntry(StorageIntf *s,Entry *e)
   marshalBool(s,e->hidden);
   marshalBool(s,e->artificial);
   marshalInt(s,(int)e->groupDocType);
+  marshalQCString(s,e->id);
+  marshalQCString(s,e->metaData);
 }
 
 void marshalEntryTree(StorageIntf *s,Entry *e)
@@ -448,6 +448,13 @@ uint unmarshalUInt(StorageIntf *s)
   return result;
 }
 
+uint64 unmarshalUInt64(StorageIntf *s)
+{
+  uint64 result=uint64(unmarshalUInt(s))<<32;
+  result|=unmarshalUInt(s);
+  return result;
+}
+
 bool unmarshalBool(StorageIntf *s)
 {
   char result;
@@ -464,7 +471,7 @@ QCString unmarshalQCString(StorageIntf *s)
   result.at(len)='\0';
   if (len>0)
   {
-    s->read(result.data(),len);
+    s->read(result.rawData(),len);
   }
   //printf("unmarshalQCString: result=%s\n",result.data());
   return result;
@@ -486,28 +493,7 @@ QGString unmarshalQGString(StorageIntf *s)
 
 ArgumentList *unmarshalArgumentList(StorageIntf *s)
 {
-  uint i;
-  uint count = unmarshalUInt(s);
-  if (count==NULL_LIST) return 0; // null list
-  ArgumentList *result = new ArgumentList;
-  assert(count<1000000);
-  //printf("unmarshalArgumentList: %d\n",count);
-  for (i=0;i<count;i++)
-  {
-    Argument *a = new Argument;
-    a->attrib  = unmarshalQCString(s);
-    a->type    = unmarshalQCString(s);
-    a->canType = unmarshalQCString(s);
-    a->name    = unmarshalQCString(s);
-    a->array   = unmarshalQCString(s);
-    a->defval  = unmarshalQCString(s);
-    a->docs    = unmarshalQCString(s);
-    result->append(a);
-  }
-  result->constSpecifier    = unmarshalBool(s);
-  result->volatileSpecifier = unmarshalBool(s);
-  result->pureSpecifier     = unmarshalBool(s);
-  return result;
+  return ArgumentList::unmarshal(s);
 }
 
 QList<ArgumentList> *unmarshalArgumentLists(StorageIntf *s)
@@ -566,19 +552,31 @@ QList<SectionInfo> *unmarshalSectionInfoList(StorageIntf *s)
   uint i;
   uint count = unmarshalUInt(s);
   if (count==NULL_LIST) return 0; // null list
-  QList<SectionInfo> *result = new QList<SectionInfo>;
-  result->setAutoDelete(TRUE);
+  QList<SectionInfo> *anchors = new QList<SectionInfo>;
   assert(count<1000000);
   for (i=0;i<count;i++)
-  { 
+  {
     QCString label = unmarshalQCString(s);
     QCString title = unmarshalQCString(s);
     QCString ref   = unmarshalQCString(s);
     SectionInfo::SectionType type = (SectionInfo::SectionType)unmarshalInt(s);
     QCString fileName = unmarshalQCString(s);
-    result->append(new SectionInfo(fileName,label,title,type,ref));
+    int lineNr = unmarshalInt(s);
+    int level = unmarshalInt(s);
+    SectionInfo *si = Doxygen::sectionDict->find(label);
+    if (si==0) // This should actually never be true since all anchors should be in sectionDict.
+               // Could still optimize the marshaling routine by only storing label.
+    {
+      SectionInfo *si = new SectionInfo(fileName,lineNr,label,title,type,level,ref);
+      anchors->append(si);
+      Doxygen::sectionDict->append(label,si); // this dict owns the anchor objects
+    }
+    else
+    {
+      anchors->append(si);
+    }
   }
-  return result;
+  return anchors;
 }
 
 QList<ListItemInfo> *unmarshalItemInfoList(StorageIntf *s)
@@ -619,7 +617,7 @@ SectionDict *unmarshalSectionDict(StorageIntf *s)
     QCString key    = unmarshalQCString(s);
     SectionInfo *si = (SectionInfo *)unmarshalObjPointer(s);
     //printf("  unmarshalSectionDict i=%d key=%s si=%s\n",count,key.data(),si->label.data());
-    result->insert(key,si);
+    result->append(key,si);
   }
   return result;
 }
@@ -644,9 +642,9 @@ MemberSDict *unmarshalMemberSDict(StorageIntf *s)
     //printf("  unmarshaling key %s\n",key.data());
     MemberDef *md = (MemberDef *)unmarshalObjPointer(s);
     //printf("  unmarshalMemberSDict i=%d key=%s md=%p\n",i,key.data(),md);
-    result->append(key,md); // note: this can lead to unmarshalling another object!
+    result->append(key,md); 
   }
-  result->sort();
+
   //printf("--- end unmarshalMemberSDict\n");
   return result;
 }
@@ -751,6 +749,33 @@ SDict<MemberList> *unmarshalMemberLists(StorageIntf *s)
   return result;
 }
 
+LocalToc unmarshalLocalToc(StorageIntf *s)
+{
+  LocalToc result;
+  int mask       = unmarshalInt(s);
+  int htmlLevel  = unmarshalInt(s);
+  int latexLevel = unmarshalInt(s);
+  int xmlLevel   = unmarshalInt(s);
+  int docbookLevel   = unmarshalInt(s);
+  if ((mask & (1<<LocalToc::Html))!=0)
+  {
+    result.enableHtml(htmlLevel);
+  }
+  if ((mask & (1<<LocalToc::Latex))!=0)
+  {
+    result.enableLatex(latexLevel);
+  }
+  if ((mask & (1<<LocalToc::Xml))!=0)
+  {
+    result.enableXml(xmlLevel);
+  }
+  if ((mask & (1<<LocalToc::Docbook))!=0)
+  {
+    result.enableDocbook(docbookLevel);
+  }
+  return result;
+}
+
 Entry * unmarshalEntry(StorageIntf *s)
 {
   Entry *e = new Entry;
@@ -761,14 +786,17 @@ Entry * unmarshalEntry(StorageIntf *s)
   e->section          = unmarshalInt(s);
   e->protection       = (Protection)unmarshalInt(s);
   e->mtype            = (MethodTypes)unmarshalInt(s);
-  e->spec             = unmarshalInt(s);
+  e->spec             = unmarshalUInt64(s);
   e->initLines        = unmarshalInt(s);
   e->stat             = unmarshalBool(s);
+  e->localToc         = unmarshalLocalToc(s);
   e->explicitExternal = unmarshalBool(s);
   e->proto            = unmarshalBool(s);
   e->subGrouping      = unmarshalBool(s);
   e->callGraph        = unmarshalBool(s);
   e->callerGraph      = unmarshalBool(s);
+  e->referencedByRelation = unmarshalBool(s);
+  e->referencesRelation   = unmarshalBool(s);
   e->virt             = (Specifier)unmarshalInt(s);
   e->args             = unmarshalQCString(s);
   e->bitfields        = unmarshalQCString(s);
@@ -811,6 +839,8 @@ Entry * unmarshalEntry(StorageIntf *s)
   e->hidden           = unmarshalBool(s);
   e->artificial       = unmarshalBool(s);
   e->groupDocType     = (Entry::GroupDocType)unmarshalInt(s);
+  e->id               = unmarshalQCString(s);
+  e->metaData         = unmarshalQCString(s);
   return e;
 }
 
