@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -42,7 +42,7 @@ MemberGroup::MemberGroup()
 }
 
 MemberGroup::MemberGroup(Definition *parent,
-      int id,const char *hdr,const char *d,const char *docFile) 
+      int id,const char *hdr,const char *d,const char *docFile,int docLine)
 {
   //printf("New member group id=%d header=%s desc=%s\n",id,hdr,d);
   memberList      = new MemberList(MemberListType_memberGroup);
@@ -56,6 +56,7 @@ MemberGroup::MemberGroup(Definition *parent,
   m_numDocMembers = -1;
   m_parent        = parent;
   m_docFile       = docFile;
+  m_docLine       = docLine;
   m_xrefListItems = 0;
   //printf("Member group docs=`%s'\n",doc.data());
 }
@@ -76,7 +77,7 @@ void MemberGroup::insertMember(MemberDef *md)
   //       md,md->name().data());
 
   MemberDef *firstMd = memberList->getFirst();
-  if (inSameSection && memberList->count()>0 && 
+  if (inSameSection && firstMd &&
       firstMd->getSectionList(m_parent)!=md->getSectionList(m_parent))
   {
     inSameSection=FALSE;
@@ -146,7 +147,8 @@ void MemberGroup::addGroupedInheritedMembers(OutputList &ol,ClassDef *cd,
   for (li.toFirst();(md=li.current());++li)
   {
     //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
-    if (lt==md->getSectionList(m_parent)->listType())
+    MemberList *ml = md->getSectionList(m_parent);
+    if (ml && lt==ml->listType())
     {
       MemberList ml(lt);
       ml.append(md);
@@ -164,7 +166,8 @@ int MemberGroup::countGroupedInheritedMembers(MemberListType lt)
   for (li.toFirst();(md=li.current());++li)
   {
     //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
-    if (lt==md->getSectionList(m_parent)->listType())
+    MemberList *ml = md->getSectionList(m_parent);
+    if (ml && lt==ml->listType())
     {
       count++;
     }
@@ -275,6 +278,16 @@ int MemberGroup::typedefCount() const
   return memberList->typedefCount(); 
 }
 
+int MemberGroup::sequenceCount() const   
+{ 
+  return memberList->sequenceCount(); 
+}
+
+int MemberGroup::dictionaryCount() const   
+{ 
+  return memberList->dictionaryCount(); 
+}
+
 int MemberGroup::protoCount() const     
 { 
   return memberList->protoCount(); 
