@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: namespacedef.h,v 1.18 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2011 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,13 +18,13 @@
 #ifndef NAMESPACEDEF_H
 #define NAMESPACEDEF_H
 
-#include "qtbc.h"
 #include <qstrlist.h>
 #include <qdict.h>
 #include "sortdict.h"
 #include "definition.h"
-#include "memberlist.h"
+#include "filedef.h"
 
+class MemberList;
 class ClassDef;
 class ClassList;
 class OutputList;
@@ -33,21 +33,26 @@ class MemberDef;
 class NamespaceList;
 class MemberGroupSDict;
 class NamespaceSDict;
+class FTextStream;
 
+/** A model of a namespace symbol. */
 class NamespaceDef : public Definition
 {
   public:
-    NamespaceDef(const char *defFileName,int defLine,
+    NamespaceDef(const char *defFileName,int defLine,int defColumn,
                  const char *name,const char *ref=0,
-                 const char *refFile=0);
+                 const char *refFile=0,const char*type=0,
+                 bool isPublished=false);
    ~NamespaceDef();
     DefType definitionType() const { return TypeNamespace; }
     QCString getOutputFileBase() const;
-    void insertUsedFile(const char *fname);
+    QCString anchor() const { return QCString(); }
+    void insertUsedFile(FileDef *fd);
     
     void writeDocumentation(OutputList &ol);
     void writeMemberPages(OutputList &ol);
     void writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const;
+    void writeTagFile(FTextStream &);
 
     void insertClass(ClassDef *cd);
     void insertNamespace(NamespaceDef *nd);
@@ -60,20 +65,29 @@ class NamespaceDef : public Definition
     void addUsingDeclaration(Definition *def);
     SDict<Definition> *getUsedClasses() const { return usingDeclList; }
     void combineUsingRelations();
-    QCString displayName() const;
-    
+    QCString displayName(bool=TRUE) const;
+    QCString localName() const;
+
+    bool isConstantGroup() const { return CONSTANT_GROUP == m_type; }
+    bool isModule()        const { return MODULE == m_type; }
+    bool isLibrary() const { return LIBRARY == m_type; }
+
     bool isLinkableInProject() const;
     bool isLinkable() const;
+    bool hasDetailedDescription() const;
     void addMembersToMemberGroup();
     void distributeMemberGroupDocumentation();
     void findSectionsInDocumentation();
     void sortMemberLists();
 
-    virtual Definition *findInnerCompound(const char *name);
+    virtual Definition *findInnerCompound(const char *name) const;
     void addInnerCompound(Definition *d);
     void addListReferences();
+    void setFileName(const QCString &fn);
+
+    bool subGrouping() const { return m_subGrouping; }
     
-    MemberList *getMemberList(MemberList::ListType lt) const;
+    MemberList *getMemberList(MemberListType lt) const;
     const QList<MemberList> &getMemberLists() const { return m_memberLists; }
     MemberDef    *getMemberByName(const QCString &) const;
 
@@ -83,31 +97,48 @@ class NamespaceDef : public Definition
     /*! Returns the classes contained in this namespace */
     ClassSDict *getClassSDict() const { return classSDict; }
 
+    /*! Returns the Slice interfaces contained in this namespace */
+    ClassSDict *getInterfaceSDict() const { return interfaceSDict; }
+
+    /*! Returns the Slice structs contained in this namespace */
+    ClassSDict *getStructSDict() const { return structSDict; }
+
+    /*! Returns the Slice exceptions contained in this namespace */
+    ClassSDict *getExceptionSDict() const { return exceptionSDict; }
+
     /*! Returns the namespaces contained in this namespace */
     NamespaceSDict *getNamespaceSDict() const { return namespaceSDict; }
 
+    QCString title() const;
+    QCString compoundTypeString() const;
+
+    void setMetaData(const QCString &m);
+
     bool visited;
 
   private:
-    MemberList *createMemberList(MemberList::ListType lt);
-    void addMemberToList(MemberList::ListType lt,MemberDef *md);
-    void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title);
-    void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title);
+    MemberList *createMemberList(MemberListType lt);
+    void addMemberToList(MemberListType lt,MemberDef *md);
+    void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title);
+    void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title);
     void writeDetailedDescription(OutputList &ol,const QCString &title);
     void writeBriefDescription(OutputList &ol);
     void startMemberDeclarations(OutputList &ol);
     void endMemberDeclarations(OutputList &ol);
-    void writeClassDeclarations(OutputList &ol,const QCString &title);
+    void writeClassDeclarations(OutputList &ol,const QCString &title,ClassSDict *d);
     void writeInlineClasses(OutputList &ol);
-    void writeNamespaceDeclarations(OutputList &ol,const QCString &title);
+    void writeNamespaceDeclarations(OutputList &ol,const QCString &title,
+            bool isConstantGroup=false);
     void writeMemberGroups(OutputList &ol);
     void writeAuthorSection(OutputList &ol);
     void startMemberDocumentation(OutputList &ol);
     void endMemberDocumentation(OutputList &ol);
     void writeSummaryLinks(OutputList &ol);
+    void addNamespaceAttributes(OutputList &ol);
+    void writeClassesToTagFile(FTextStream &,ClassSDict *d);
 
     QCString              fileName;
-    QStrList              files;
+    FileList              files;
 
     NamespaceSDict       *usingDirList;
     SDict<Definition>    *usingDeclList;
@@ -117,22 +148,28 @@ class NamespaceDef : public Definition
     QList<MemberList>     m_memberLists;
     MemberGroupSDict     *memberGroupSDict;
     ClassSDict           *classSDict;
+    ClassSDict           *interfaceSDict;
+    ClassSDict           *structSDict;
+    ClassSDict           *exceptionSDict;
     NamespaceSDict       *namespaceSDict;
     bool                  m_subGrouping;
+    enum { NAMESPACE, MODULE, CONSTANT_GROUP, LIBRARY } m_type;
+    bool m_isPublished;
+    QCString              metaData;
 };
 
+/** A list of NamespaceDef objects. */
 class NamespaceList : public QList<NamespaceDef>
-{ 
+{
   public:
    ~NamespaceList() {}
-    int compareItems(GCI item1,GCI item2)
+    int compareValues(const NamespaceDef *nd1,const NamespaceDef *nd2) const
     {
-      return stricmp(((NamespaceDef *)item1)->name(),
-                    ((NamespaceDef *)item2)->name()
-                   );
+      return qstricmp(nd1->name(), nd2->name());
     }
 };
 
+/** An iterator for NamespaceDef objects in a NamespaceList. */
 class NamespaceListIterator : public QListIterator<NamespaceDef>
 {
   public:
@@ -140,6 +177,7 @@ class NamespaceListIterator : public QListIterator<NamespaceDef>
       QListIterator<NamespaceDef>(l) {}
 };
 
+/** An unsorted dictionary of NamespaceDef objects. */
 class NamespaceDict : public QDict<NamespaceDef>
 {
   public:
@@ -147,19 +185,20 @@ class NamespaceDict : public QDict<NamespaceDef>
    ~NamespaceDict() {}
 };
 
+/** A sorted dictionary of NamespaceDef objects. */
 class NamespaceSDict : public SDict<NamespaceDef>
 {
   public:
     NamespaceSDict(int size=17) : SDict<NamespaceDef>(size) {}
    ~NamespaceSDict() {}
-    int compareItems(GCI item1,GCI item2)
-    {
-      return stricmp(((NamespaceDef *)item1)->name(),
-                    ((NamespaceDef *)item2)->name()
-                   );
-    }
-    void writeDeclaration(OutputList &ol,const char *title,bool localName=FALSE);
+    void writeDeclaration(OutputList &ol,const char *title,
+            bool isConstantGroup=false, bool localName=FALSE);
     bool declVisible() const;
+  private:
+    int compareValues(const NamespaceDef *item1,const NamespaceDef *item2) const
+    {
+      return qstricmp(item1->name(),item2->name());
+    }
 };
 
 
