@@ -3,7 +3,7 @@
  * 
  *
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -34,6 +34,7 @@
 #include "filedef.h"
 #include "config.h"
 #include "htmlentity.h"
+#include "emoji.h"
 #include "plantuml.h"
 
 //#define DBG_RTF(x) m_t << x
@@ -136,11 +137,53 @@ void RTFDocVisitor::visit(DocSymbol *s)
   m_lastIsPara=FALSE;
 }
 
+void RTFDocVisitor::visit(DocEmoji *s)
+{
+  if (m_hide) return;
+  DBG_RTF("{\\comment RTFDocVisitor::visit(DocEmoji)}\n");
+  const char *res = EmojiEntityMapper::instance()->unicode(s->index());
+  if (res)
+  {
+    const char *p = res;
+    int val = 0;
+    int val1 = 0;
+    while (*p)
+    {
+      switch(*p)
+      {
+        case '&': case '#': case 'x':
+          break;
+        case ';':
+	  val1 = val;
+	  val = 0xd800 + ( ( val1 - 0x10000 ) & 0xffc00 ) / 0x400 - 0x10000;
+          m_t << "\\u" << val << "?";
+          val = 0xdC00 + ( ( val1 - 0x10000 ) & 0x3ff ) - 0x10000 ;
+          m_t << "\\u" << val << "?";
+          val = 0;
+          break;
+        case '0': case '1': case '2': case '3': case '4':
+        case '5': case '6': case '7': case '8': case '9':
+          val = val * 16 + *p - '0';
+          break;
+        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
+          val = val * 16 + *p - 'a' + 10;
+          break;
+      }
+      p++;
+    }
+  }
+  else
+  {
+    m_t << s->name();
+  }
+  m_lastIsPara=FALSE;
+}
+
 void RTFDocVisitor::visit(DocURL *u)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visit(DocURL)}\n");
-  if (Config_getBool("RTF_HYPERLINKS"))
+  if (Config_getBool(RTF_HYPERLINKS))
   {
     m_t << "{\\field "
              "{\\*\\fldinst "
@@ -192,6 +235,12 @@ void RTFDocVisitor::visit(DocStyleChange *s)
     case DocStyleChange::Bold:
       if (s->enable()) m_t << "{\\b ";      else m_t << "} ";
       break;
+    case DocStyleChange::Strike:
+      if (s->enable()) m_t << "{\\strike ";      else m_t << "} ";
+      break;
+    case DocStyleChange::Underline:
+      if (s->enable()) m_t << "{\\ul ";      else m_t << "} ";
+      break;
     case DocStyleChange::Italic:
       if (s->enable()) m_t << "{\\i ";     else m_t << "} ";
       break;
@@ -231,6 +280,13 @@ void RTFDocVisitor::visit(DocStyleChange *s)
   }
 }
 
+static void visitCaption(RTFDocVisitor *parent, QList<DocNode> children)
+{
+  QListIterator<DocNode> cli(children);
+  DocNode *n;
+  for (cli.toFirst();(n=cli.current());++cli) n->accept(parent);
+}
+
 void RTFDocVisitor::visit(DocVerbatim *s)
 {
   if (m_hide) return;
@@ -277,7 +333,7 @@ void RTFDocVisitor::visit(DocVerbatim *s)
         QCString fileName(4096);
 
         fileName.sprintf("%s%d%s", 
-            (Config_getString("RTF_OUTPUT")+"/inline_dotgraph_").data(), 
+            (Config_getString(RTF_OUTPUT)+"/inline_dotgraph_").data(), 
             dotindex++,
             ".dot"
            );
@@ -288,10 +344,12 @@ void RTFDocVisitor::visit(DocVerbatim *s)
         }
         file.writeBlock( s->text(), s->text().length() );
         file.close();
-        m_t << "\\par{\\qc "; // center picture
-        writeDotFile(fileName);
-        m_t << "} ";
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
+
+        writeDotFile(fileName, s->hasCaption());
+        visitCaption(this, s->children());
+        includePicturePostRTF(true, s->hasCaption());
+
+        if (Config_getBool(DOT_CLEANUP)) file.remove();
       }
       break;
     case DocVerbatim::Msc: 
@@ -300,7 +358,7 @@ void RTFDocVisitor::visit(DocVerbatim *s)
         QCString baseName(4096);
 
         baseName.sprintf("%s%d%s",
-            (Config_getString("RTF_OUTPUT")+"/inline_mscgraph_").data(), 
+            (Config_getString(RTF_OUTPUT)+"/inline_mscgraph_").data(), 
             mscindex++,
             ".msc"
            );
@@ -314,20 +372,22 @@ void RTFDocVisitor::visit(DocVerbatim *s)
         text+="}";
         file.writeBlock( text, text.length() );
         file.close();
-        m_t << "\\par{\\qc "; // center picture
-        writeMscFile(baseName);
-        m_t << "} ";
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
+
+        writeMscFile(baseName, s->hasCaption());
+        visitCaption(this, s->children());
+        includePicturePostRTF(true, s->hasCaption());
+
+        if (Config_getBool(DOT_CLEANUP)) file.remove();
       }
       break;
     case DocVerbatim::PlantUML:
       {
-        static QCString rtfOutput = Config_getString("RTF_OUTPUT");
+        static QCString rtfOutput = Config_getString(RTF_OUTPUT);
         QCString baseName = writePlantUMLSource(rtfOutput,s->exampleFile(),s->text());
 
-        m_t << "\\par{\\qc "; // center picture
-        writePlantUMLFile(baseName);
-        m_t << "} ";
+        writePlantUMLFile(baseName, s->hasCaption());
+        visitCaption(this, s->children());
+        includePicturePostRTF(true, s->hasCaption());
       }
       break;
   }
@@ -375,7 +435,14 @@ void RTFDocVisitor::visit(DocInclude *inc)
                                            inc->text(),
                                            langExt,
                                            inc->isExample(),
-                                           inc->exampleFile(), &fd);
+                                           inc->exampleFile(),
+                                           &fd,   // fileDef,
+                                           -1,    // start line
+                                           -1,    // end line
+                                           FALSE, // inline fragment
+                                           0,     // memberDef
+                                           TRUE   // show line numbers
+					   );
          m_t << "\\par";
          m_t << "}" << endl;
       }
@@ -387,7 +454,14 @@ void RTFDocVisitor::visit(DocInclude *inc)
       Doxygen::parserManager->getParser(inc->extension())
                             ->parseCode(m_ci,inc->context(),
                                         inc->text(),langExt,inc->isExample(),
-                                        inc->exampleFile());
+                                        inc->exampleFile(),
+                                        0,     // fileDef
+                                        -1,    // startLine
+                                        -1,    // endLine
+                                        TRUE,  // inlineFragment
+                                        0,     // memberDef
+                                        FALSE  // show line numbers
+				       );
       m_t << "\\par";
       m_t << "}" << endl;
       break;
@@ -419,6 +493,35 @@ void RTFDocVisitor::visit(DocInclude *inc)
                                        );
       m_t << "}";
       break;
+    case DocInclude::SnipWithLines:
+      {
+         QFileInfo cfi( inc->file() );
+         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
+         m_t << "{" << endl;
+         if (!m_lastIsPara) m_t << "\\par" << endl;
+         m_t << rtf_Style_Reset << getStyle("CodeExample");
+         Doxygen::parserManager->getParser(inc->extension())
+                               ->parseCode(m_ci,
+                                           inc->context(),
+                                           extractBlock(inc->text(),inc->blockId()),
+                                           langExt,
+                                           inc->isExample(),
+                                           inc->exampleFile(), 
+                                           &fd,
+                                           lineBlock(inc->text(),inc->blockId()),
+                                           -1,    // endLine
+                                           FALSE, // inlineFragment
+                                           0,     // memberDef
+                                           TRUE   // show line number
+                                          );
+         m_t << "}";
+      }
+      break;
+    case DocInclude::SnippetDoc: 
+    case DocInclude::IncludeDoc: 
+      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
+          "Please create a bug report\n",__FILE__);
+      break;
   }
   m_lastIsPara=TRUE;
 }
@@ -472,9 +575,21 @@ void RTFDocVisitor::visit(DocIncOperator *op)
 void RTFDocVisitor::visit(DocFormula *f)
 {
   if (m_hide) return;
-  // TODO: do something sensible here, like including a bitmap
   DBG_RTF("{\\comment RTFDocVisitor::visit(DocFormula)}\n");
-  m_t << f->text();
+  bool bDisplay = !f->isInline();
+  if (bDisplay) 
+  {
+    m_t << "\\par";
+    m_t << "{";
+    m_t << "\\pard\\plain";
+    m_t << "\\pard";
+    m_t << "\\qc";
+  }
+  m_t << "{ \\field\\flddirty {\\*\\fldinst  INCLUDEPICTURE \"" << f->relPath() << f->name() << ".png\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt Image}}";
+  if (bDisplay) 
+  {
+    m_t << "\\par}";
+  }
   m_lastIsPara=FALSE;
 }
 
@@ -532,9 +647,10 @@ void RTFDocVisitor::visitPost(DocAutoList *)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocAutoList)}\n");
-  m_t << "\\par";
+  if (!m_lastIsPara) m_t << "\\par";
   m_t << "}" << endl;
   m_lastIsPara=TRUE;
+  if (!m_indentLevel) m_t << "\\par";
 }
 
 void RTFDocVisitor::visitPre(DocAutoListItem *)
@@ -958,7 +1074,7 @@ void RTFDocVisitor::visitPre(DocHRef *href)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocHRef)}\n");
-  if (Config_getBool("RTF_HYPERLINKS"))
+  if (Config_getBool(RTF_HYPERLINKS))
   {
     m_t << "{\\field "
              "{\\*\\fldinst "
@@ -980,7 +1096,7 @@ void RTFDocVisitor::visitPost(DocHRef *)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocHRef)}\n");
-  if (Config_getBool("RTF_HYPERLINKS"))
+  if (Config_getBool(RTF_HYPERLINKS))
   { 
     m_t <<     "}"
              "}"
@@ -1000,21 +1116,21 @@ void RTFDocVisitor::visitPre(DocHtmlHeader *header)
   m_t << "{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = QMIN(header->level()+2,4);
+  int level = QMIN(header->level(),5);
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading]->reference;
-  // make table of contents entry
-  m_t << "{\\tc\\tcl \\v " << level << "}";
+  // make open table of contents entry that will be closed in visitPost method
+  m_t << "{\\tc\\tcl" << level << " ";
   m_lastIsPara=FALSE;
-  
 }
 
 void RTFDocVisitor::visitPost(DocHtmlHeader *) 
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocHtmlHeader)}\n");
-  m_t << "\\par";
+  // close open table of contents entry
+  m_t << "} \\par";
   m_t << "}" << endl; // end section
   m_lastIsPara=TRUE;
 }
@@ -1022,76 +1138,112 @@ void RTFDocVisitor::visitPost(DocHtmlHeader *)
 void RTFDocVisitor::visitPre(DocImage *img)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocImage)}\n");
-  if (img->type()==DocImage::Rtf)
+  includePicturePreRTF(img->name(), img->type()==DocImage::Rtf, img->hasCaption(), img->isInlineImage());
+}
+void RTFDocVisitor::includePicturePreRTF(const QCString name, bool isTypeRTF, bool hasCaption, bool inlineImage)
+{
+  if (isTypeRTF)
   {
-    m_t << "\\par" << endl;
-    m_t << "{" << endl;
-    m_t << rtf_Style_Reset << endl;
-    m_t << "\\par\\pard \\qc {\\field\\flddirty {\\*\\fldinst INCLUDEPICTURE \"";
-    m_t << img->name();
-    m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt IMAGE}}\\par" << endl;
-    m_t << "}" << endl;
-    m_lastIsPara=TRUE;
+    if (!inlineImage)
+    {
+      m_t << "\\par" << endl;
+      m_t << "{" << endl;
+      m_t << rtf_Style_Reset << endl;
+      if (hasCaption || m_lastIsPara) m_t << "\\par" << endl;
+      m_t << "\\pard \\qc ";
+    }
+    m_t << "{ \\field\\flddirty {\\*\\fldinst  INCLUDEPICTURE \"";
+    m_t << name;
+    m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt Image}}" << endl;
+    if (!inlineImage)
+    {
+      m_t << "\\par" << endl;
+      if (hasCaption)
+      {
+         m_t << "\\pard \\qc \\b";
+         m_t << "{Image \\field\\flddirty{\\*\\fldinst { SEQ Image \\\\*Arabic }}{\\fldrslt {\\noproof 1}} ";
+      }
+      m_lastIsPara=TRUE;
+    }
+    else
+    {
+      if (hasCaption) m_t << "{\\comment "; // to prevent caption to be shown
+    }
   }
   else // other format -> skip
   {
+    pushEnabled();
+    m_hide=TRUE;
   }
-  // hide caption since it is not supported at the moment
-  pushEnabled();
-  m_hide=TRUE;
 }
 
-void RTFDocVisitor::visitPost(DocImage *) 
+void RTFDocVisitor::visitPost(DocImage *img)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocImage)}\n");
-  popEnabled();
+  includePicturePostRTF(img->type()==DocImage::Rtf, img->hasCaption(), img->isInlineImage());
+}
+
+void RTFDocVisitor::includePicturePostRTF(bool isTypeRTF, bool hasCaption, bool inlineImage)
+{
+  if (isTypeRTF)
+  {
+    if (m_hide) return;
+    if (inlineImage)
+    {
+      if (hasCaption) m_t << " }";
+    }
+    else
+    {
+      if (hasCaption)
+      {
+        m_t << "}" <<endl;
+        m_t << "\\par}" <<endl;
+      }
+      else
+      {
+        m_t << "}" <<endl;
+      }
+    }
+  }
+  else
+  {
+    popEnabled();
+  }
 }
 
 void RTFDocVisitor::visitPre(DocDotFile *df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocDotFile)}\n");
-  writeDotFile(df->file());
-
-  // hide caption since it is not supported at the moment
-  pushEnabled();
-  m_hide=TRUE;
+  writeDotFile(df);
 }
 
-void RTFDocVisitor::visitPost(DocDotFile *) 
+void RTFDocVisitor::visitPost(DocDotFile *df) 
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocDotFile)}\n");
-  popEnabled();
+  includePicturePostRTF(true, df->hasCaption());
 }
 void RTFDocVisitor::visitPre(DocMscFile *df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocMscFile)}\n");
-  writeMscFile(df->file());
-
-  // hide caption since it is not supported at the moment
-  pushEnabled();
-  m_hide=TRUE;
+  writeMscFile(df);
 }
 
-void RTFDocVisitor::visitPost(DocMscFile *) 
+void RTFDocVisitor::visitPost(DocMscFile *df) 
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocMscFile)}\n");
-  popEnabled();
+  includePicturePostRTF(true, df->hasCaption());
 }
 
 void RTFDocVisitor::visitPre(DocDiaFile *df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocDiaFile)}\n");
-  writeDiaFile(df->file());
-
-  // hide caption since it is not supported at the moment
-  pushEnabled();
-  m_hide=TRUE;
+  writeDiaFile(df);
 }
 
-void RTFDocVisitor::visitPost(DocDiaFile *)
+void RTFDocVisitor::visitPost(DocDiaFile *df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocDiaFile)}\n");
-  popEnabled();
+  includePicturePostRTF(true, df->hasCaption());
 }
 
 void RTFDocVisitor::visitPre(DocLink *lnk)
@@ -1168,7 +1320,7 @@ void RTFDocVisitor::visitPost(DocSecRefList *)
 //void RTFDocVisitor::visitPre(DocLanguage *l)
 //{
 //  DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocLanguage)}\n");
-//  QCString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QCString langId = Config_getEnum(OUTPUT_LANGUAGE);
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    pushEnabled();
@@ -1179,7 +1331,7 @@ void RTFDocVisitor::visitPost(DocSecRefList *)
 //void RTFDocVisitor::visitPost(DocLanguage *l) 
 //{
 //  DBG_RTF("{\\comment RTFDocVisitor::visitPost(DocLanguage)}\n");
-//  QCString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QCString langId = Config_getEnum(OUTPUT_LANGUAGE);
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    popEnabled();
@@ -1203,10 +1355,7 @@ void RTFDocVisitor::visitPre(DocParamSect *s)
     case DocParamSect::Exception: 
       m_t << theTranslator->trExceptions(); break;
     case DocParamSect::TemplateParam: 
-      /* TODO: add this 
-      m_t << theTranslator->trTemplateParam(); break;
-      */
-      m_t << "Template Parameters"; break;
+      m_t << theTranslator->trTemplateParameters(); break;
     default:
       ASSERT(0);
   }
@@ -1414,10 +1563,15 @@ void RTFDocVisitor::visitPre(DocXRefItem *x)
   if (x->title().isEmpty()) return;
   bool anonymousEnum = x->file()=="@";
   DBG_RTF("{\\comment RTFDocVisitor::visitPre(DocXRefItem)}\n");
+  if (!m_lastIsPara)
+  {
+    m_t << "\\par" << endl;
+    m_lastIsPara=TRUE;
+  }
   m_t << "{"; // start param list
   //m_t << "{\\b "; // start bold
   m_t << "{" << rtf_Style["Heading5"]->reference << endl;
-  if (Config_getBool("RTF_HYPERLINKS") && !anonymousEnum)
+  if (Config_getBool(RTF_HYPERLINKS) && !anonymousEnum)
   {
     QCString refName;
     if (!x->file().isEmpty())
@@ -1603,7 +1757,7 @@ void RTFDocVisitor::filter(const char *str,bool verbatim)
 
 void RTFDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
 {
-  if (ref.isEmpty() && Config_getBool("RTF_HYPERLINKS"))
+  if (ref.isEmpty() && Config_getBool(RTF_HYPERLINKS))
   {
     QCString refName;
     if (!file.isEmpty())
@@ -1633,7 +1787,7 @@ void RTFDocVisitor::startLink(const QCString &ref,const QCString &file,const QCS
 
 void RTFDocVisitor::endLink(const QCString &ref)
 {
-  if (ref.isEmpty() && Config_getBool("RTF_HYPERLINKS"))
+  if (ref.isEmpty() && Config_getBool(RTF_HYPERLINKS))
   {
     m_t << "}}}";
   }
@@ -1657,27 +1811,29 @@ void RTFDocVisitor::popEnabled()
   delete v;
 }
 
-void RTFDocVisitor::writeDotFile(const QCString &fileName)
+void RTFDocVisitor::writeDotFile(DocDotFile *df)
 {
-  QCString baseName=fileName;
+  writeDotFile(df->file(), df->hasCaption());
+}
+void RTFDocVisitor::writeDotFile(const QCString &filename, bool hasCaption)
+{
+  QCString baseName=filename;
   int i;
   if ((i=baseName.findRev('/'))!=-1)
   {
     baseName=baseName.right(baseName.length()-i-1);
   } 
-  QCString outDir = Config_getString("RTF_OUTPUT");
-  writeDotGraphFromFile(fileName,outDir,baseName,GOF_BITMAP);
-  if (!m_lastIsPara) m_t << "\\par" << endl;
-  m_t << "{" << endl;
-  m_t << rtf_Style_Reset;
-  m_t << "\\pard \\qc {\\field\\flddirty {\\*\\fldinst INCLUDEPICTURE \"";
-  m_t << baseName << "." << Config_getEnum("DOT_IMAGE_FORMAT");
-  m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt IMAGE}}\\par" << endl;
-  m_t << "}" << endl;
-  m_lastIsPara=TRUE;
+  QCString outDir = Config_getString(RTF_OUTPUT);
+  writeDotGraphFromFile(filename,outDir,baseName,GOF_BITMAP);
+  QCString imgExt = getDotImageExtension();
+  includePicturePreRTF(baseName + "." + imgExt, true, hasCaption);
 }
 
-void RTFDocVisitor::writeMscFile(const QCString &fileName)
+void RTFDocVisitor::writeMscFile(DocMscFile *df)
+{
+  writeMscFile(df->file(), df->hasCaption());
+}
+void RTFDocVisitor::writeMscFile(const QCString &fileName, bool hasCaption)
 {
   QCString baseName=fileName;
   int i;
@@ -1685,19 +1841,25 @@ void RTFDocVisitor::writeMscFile(const QCString &fileName)
   {
     baseName=baseName.right(baseName.length()-i-1);
   } 
-  QCString outDir = Config_getString("RTF_OUTPUT");
+  QCString outDir = Config_getString(RTF_OUTPUT);
   writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP);
-  if (!m_lastIsPara) m_t << "\\par" << endl;
-  m_t << "{" << endl;
-  m_t << rtf_Style_Reset;
-  m_t << "\\pard \\qc {\\field\\flddirty {\\*\\fldinst INCLUDEPICTURE \"";
-  m_t << baseName << ".png";
-  m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt IMAGE}}\\par" << endl;
-  m_t << "}" << endl;
-  m_lastIsPara=TRUE;
+  includePicturePreRTF(baseName + ".png", true, hasCaption);
 }
 
-void RTFDocVisitor::writeDiaFile(const QCString &fileName)
+void RTFDocVisitor::writeDiaFile(DocDiaFile *df)
+{
+  QCString baseName=df->file();
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  }
+  QCString outDir = Config_getString(RTF_OUTPUT);
+  writeDiaGraphFromFile(df->file(),outDir,baseName,DIA_BITMAP);
+  includePicturePreRTF(baseName + ".png", true, df->hasCaption());
+}
+
+void RTFDocVisitor::writePlantUMLFile(const QCString &fileName, bool hasCaption)
 {
   QCString baseName=fileName;
   int i;
@@ -1705,35 +1867,7 @@ void RTFDocVisitor::writeDiaFile(const QCString &fileName)
   {
     baseName=baseName.right(baseName.length()-i-1);
   }
-  QCString outDir = Config_getString("RTF_OUTPUT");
-  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP);
-  if (!m_lastIsPara) m_t << "\\par" << endl;
-  m_t << "{" << endl;
-  m_t << rtf_Style_Reset;
-  m_t << "\\pard \\qc {\\field\\flddirty {\\*\\fldinst INCLUDEPICTURE \"";
-  m_t << baseName << ".png";
-  m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt IMAGE}}\\par" << endl;
-  m_t << "}" << endl;
-  m_lastIsPara=TRUE;
-}
-
-void RTFDocVisitor::writePlantUMLFile(const QCString &fileName)
-{
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  QCString outDir = Config_getString("RTF_OUTPUT");
+  QCString outDir = Config_getString(RTF_OUTPUT);
   generatePlantUMLOutput(fileName,outDir,PUML_BITMAP);
-  if (!m_lastIsPara) m_t << "\\par" << endl;
-  m_t << "{" << endl;
-  m_t << rtf_Style_Reset;
-  m_t << "\\pard \\qc {\\field\\flddirty {\\*\\fldinst INCLUDEPICTURE \"";
-  m_t << baseName << ".png";
-  m_t << "\" \\\\d \\\\*MERGEFORMAT}{\\fldrslt IMAGE}}\\par" << endl;
-  m_t << "}" << endl;
-  m_lastIsPara=TRUE;
+  includePicturePreRTF(baseName + ".png", true, hasCaption);
 }
-
