@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2014 by Parker Waechter & Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Parker Waechter & Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -62,7 +62,7 @@ class RTFGenerator : public OutputGenerator
     void endTitle() {} 
 
     void newParagraph();
-    void startParagraph();
+    void startParagraph(const char *classDef);
     void endParagraph();
     void writeString(const char *text);
     void startIndexListItem();
@@ -103,7 +103,7 @@ class RTFGenerator : public OutputGenerator
     void endMemberSections() {} 
     void startHeaderSection() {}
     void endHeaderSection() {}
-    void startMemberHeader(const char *) { startGroupHeader(FALSE); }
+    void startMemberHeader(const char *,int) { startGroupHeader(FALSE); }
     void endMemberHeader() { endGroupHeader(FALSE); }
     void startMemberSubtitle(); 
     void endMemberSubtitle(); 
@@ -120,6 +120,7 @@ class RTFGenerator : public OutputGenerator
     void startMemberTemplateParams() {}
     void endMemberTemplateParams(const char *,const char *) {}
     void insertMemberAlign(bool) {}
+    void insertMemberAlignLeft(int,bool){}
 
     void writeRuler() { rtfwriteRuler_thin(); }
 	
@@ -138,7 +139,7 @@ class RTFGenerator : public OutputGenerator
     void startDescItem();
     void endDescItem();
     void lineBreak(const char *style=0);
-    void startMemberDoc(const char *,const char *,const char *,const char *,bool);
+    void startMemberDoc(const char *,const char *,const char *,const char *,int,int,bool);
     void endMemberDoc(bool);
     void startDoxyAnchor(const char *,const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *);
@@ -156,15 +157,15 @@ class RTFGenerator : public OutputGenerator
     void startSmall()       { t << "{\\sub "; }
     void endSmall()         { t << "}"; }
 
-    void startMemberDescription(const char *,const char *);
+    void startMemberDescription(const char *,const char *,bool);
     void endMemberDescription();
     void startMemberDeclaration() {} 
     void endMemberDeclaration(const char *,const char *) {}
     void writeInheritedSectionTitle(const char *,const char *,const char *,
                       const char *,const char *,const char *) {}
     void startDescList(SectionTypes);
-    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
-    void endSimpleSect();
+    void startExamples();
+    void endExamples();
     void startParamList(ParamListTypes,const char *);
     void endParamList();
     //void writeDescItem();
@@ -193,6 +194,8 @@ class RTFGenerator : public OutputGenerator
 	
     void startDescTable(const char *title);
     void endDescTable();
+    void startDescTableRow();
+    void endDescTableRow();
     void startDescTableTitle();
     void endDescTableTitle();
     void startDescTableData();
@@ -242,8 +245,8 @@ class RTFGenerator : public OutputGenerator
     void endConstraintDocs();
     void endConstraintList();
 
-    void startMemberDocSimple();
-    void endMemberDocSimple();
+    void startMemberDocSimple(bool);
+    void endMemberDocSimple(bool);
     void startInlineMemberType();
     void endInlineMemberType();
     void startInlineMemberName();
@@ -255,8 +258,8 @@ class RTFGenerator : public OutputGenerator
     void writeLabel(const char *l,bool isLast);
     void endLabels();
 
-    void startFontClass(const char *) {}
-    void endFontClass() {}
+    void startFontClass(const char *);
+    void endFontClass();
 
     void writeCodeAnchor(const char *) {}
     void setCurrentDoc(Definition *,const char *,bool) {}
